//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "EngineRequest.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)
using namespace std;

//---------------------------------------------------------------------------
EngineRequest::EngineRequest(SocketObj *inS) : s(inS)
{
}

//---------------------------------------------------------------------------
EngineRequest::EngineRequest()// : s(SocketObj(PagesDlg->ClientSocket1))
{
        s = new SocketObj(PagesDlg->ClientSocket1);
        s->Open();
        s->Close();
}

//---------------------------------------------------------------------------
EngineRequest::~EngineRequest()
{
        if (s)
        {
                delete s;
                s = (EngineRequest *) 0;
        }
}

//---------------------------------------------------------------------------
void EngineRequest::Request(NameValueTyp &r) throw(exception)
{
        if (r.find("sysRequest") == r.end())
                throw runtime_error("EngineRequest::Request() - missing sysRequest parameter");

        PagesDlg->Animator1->Visible = true;
        PagesDlg->StatusBar1->SimpleText = "  Loading data from Engine...";
        PagesDlg->DisableAll();
        r["sysClientId"] = "RuntimeMonitor";
        r["sysReply"] = "True";
        r["sysDebug"] = "True";
        s->Open();
        (*s) << r;
        r.clear();
        (*s) >> r;
        s->Close();

        PagesDlg->EnableAll();
        PagesDlg->Animator1->Visible = false;
        PagesDlg->StatusBar1->SimpleText = "";
        if (r.find("sysResultStatus") != r.end() &&
            r["sysResultStatus"] == "-1")
        {
                if (r.find("sysResultString") != r.end())
                        throw runtime_error("EngineRequest::Request() - " +
                                        r["sysResultString"]);
                else
                        throw runtime_error("EngineRequest::Request() - " +
                                        string("Unspecified error"));
        }
}

//---------------------------------------------------------------------------
void EngineRequest::LockEngine() throw(exception)
{
        //-------------------------------------------------------------------
        // Attempt to lock the engine for us
        r.clear();
        r["sysRequest"] = "Lock";
        Request(r);
}

//---------------------------------------------------------------------------
void EngineRequest::GetEngineStatus() throw(exception)
{
        //-------------------------------------------------------------------
        // Get the engine status before displaying the main console
        r.clear();
        r["sysRequest"] = "Get Engine State";

        Request(r);

        if (r["sysConsoleLock"] == "True")
        {
                throw runtime_error("Console is Locked by " +
                        r["sysConsoleLockBy"]);
        }
        else if (r["sysEngineState"] == "Run")
        {
                PagesDlg->Button6->Caption = "Stop Engine";
                PagesDlg->Label3->Caption = (AnsiString("Running since ")
                        + AnsiString(r["sysStartDate"].c_str()));
                PagesDlg->ImageList2->GetBitmap(3, PagesDlg->Image2->Picture->Bitmap);
                PagesDlg->Image2->Refresh();
        }
        else
        {
                PagesDlg->Button6->Caption = "Start Engine";
                PagesDlg->Label3->Caption = (AnsiString("Halted since ")
                        + AnsiString(r["sysStopDate"].c_str()));
                PagesDlg->ImageList2->GetBitmap(1, PagesDlg->Image2->Picture->Bitmap);
                PagesDlg->Image2->Refresh();

        }
        PagesDlg->Label4->Caption = AnsiString(r["sysServletCount"].c_str());
        PagesDlg->Label15->Caption =AnsiString(r["sysTransFailed"].c_str());
        PagesDlg->Label16->Caption =AnsiString(r["sysTransCount"].c_str());
        PagesDlg->Label14->Caption =AnsiString(r["sysTransCompleted"].c_str());

        PagesDlg->CheckBox6->Checked = r["sysEncryption"] == "True";
        PagesDlg->CheckBox1->Checked = r["sysTransactionMode"] == "True";
        PagesDlg->CheckBox2->Checked = r["sysAudit"] == "True";
        PagesDlg->CheckBox3->Checked = r["sysRestartConnectors"] == "True";
        PagesDlg->CheckBox4->Checked = r["sysEmailOnFailure"] == "True";
        PagesDlg->Edit2->Text = AnsiString(r["sysEmailAddress"].c_str());
        PagesDlg->CheckBox5->Checked = r["sysHaltOnFailure"] == "True";
        PagesDlg->Edit1->Text= AnsiString(r["sysNumbOfFailures"].c_str());
}

//---------------------------------------------------------------------------
void EngineRequest::GetTransactionModels() throw(exception)
{
        //-------------------------------------------------------------------
        // get a list of the know transaction models
        r.clear();
        r["sysRequest"] = "List Models";

        Request(r);
        PagesDlg->AdvStringGrid3->FixedRows = 1;
        PagesDlg->AdvStringGrid3->Cols[0]->Clear();
        PagesDlg->AdvStringGrid3->Cols[1]->Clear();
        PagesDlg->AdvStringGrid3->Cols[2]->Clear();
        PagesDlg->AdvStringGrid3->Cols[3]->Clear();
        PagesDlg->AdvStringGrid3->RowCount = 2;

        PagesDlg->AdvStringGrid3->Cols[0]->Add("State");
        PagesDlg->AdvStringGrid3->Cols[1]->Add("Model Name");
        PagesDlg->AdvStringGrid3->Cols[2]->Add("Model File");
        PagesDlg->AdvStringGrid3->Cols[3]->Add("Run");


        char ndx[256];
        AnsiString modelState, modelName, modelFile, timesRun;
        for (int i = 0; sprintf(ndx, "%d", i) &&
                r.find(string("sysModelName") + ndx) != r.end(); i++)
        {
                modelName = r[string("sysModelName") + ndx].c_str();
                modelFile = r[string("sysModelFile") + ndx].c_str();
                modelState = r[string("sysModelState") + ndx].c_str();
                timesRun = r[string("sysModelTimesRun") + ndx].c_str();
                if (modelState == "Active")
                        PagesDlg->AdvStringGrid3->Cols[0]->Add("<IMG src=\"idx:0\" align=\"absmiddle\">");
                else
                        PagesDlg->AdvStringGrid3->Cols[0]->Add("<IMG src=\"idx:4\" align=\"absmiddle\">");
                PagesDlg->AdvStringGrid3->Cols[1]->Add(modelName);
                PagesDlg->AdvStringGrid3->Cols[2]->Add(modelFile);
                PagesDlg->AdvStringGrid3->Cols[3]->Add(timesRun);
                PagesDlg->AdvStringGrid3->RowCount++;
        }
        PagesDlg->AdvStringGrid3->RowCount--;
}

//---------------------------------------------------------------------------
void EngineRequest::ListConnectors() throw(exception)
{
 NameValueTyp req;
       //-------------------------------------------------------------------
       // get a list of registered connectors

        char buff[256];
        req.clear();
        req["sysRequest"] = "List Connectors";

        Request(req);

        PagesDlg->ListView1->Items->Clear();
        AnsiString connectorName;
        for (int i = 0; sprintf(buff, "Connector%d", i) && (req.find(buff) != req.end()); i++)
        {
                connectorName = req[buff].c_str();
                ListItem = PagesDlg->ListView1->Items->Add();
                ListItem->Caption = connectorName;
                GetConfigValues(ListItem, PagesDlg->AdvStringGrid2);
                if (!PagesDlg->ListView1->Selected)
                        PagesDlg->ListView1->Selected = ListItem;
        }
        if (PagesDlg->ListView1->Selected)
                GetConfigValues(PagesDlg->ListView1->Selected->Caption.c_str(),
                        PagesDlg->AdvStringGrid2);
}

//---------------------------------------------------------------------------
void EngineRequest::ListTransformers() throw(exception)
{
       //-------------------------------------------------------------------
       // get a list of registered transformers

        NameValueTyp req;
        char buff[128];
        req.clear();
        req["sysRequest"] = "List Transformers";

        Request(req);

        PagesDlg->ListView3->Items->Clear();
        AnsiString transformerName;
        for (int i = 0; sprintf(buff, "Transformer%d", i) && (req.find(buff) != req.end()); i++)
        {
                transformerName = req[buff].c_str();
                ListItem = PagesDlg->ListView3->Items->Add();
                ListItem->Caption = transformerName;
                GetConfigValues(ListItem, PagesDlg->AdvStringGrid1);
                if (!PagesDlg->ListView3->Selected)
                        PagesDlg->ListView3->Selected = ListItem;
        }

        if (PagesDlg->ListView3->Selected)
           GetConfigValues(PagesDlg->ListView3->Selected->Caption.c_str(),
                       PagesDlg->AdvStringGrid1);
}

//---------------------------------------------------------------------------
void EngineRequest::GetFailedTransactions() throw(exception)
{
 extern bool engineRunning;
        //-------------------------------------------------------------------
        // get the list of failed transactions
        r.clear();
        r["sysRequest"] = "List Failed Trans";

        Request(r);
        PagesDlg->ListView2->Items->Clear();
        AnsiString transId, transTimeStamp, transModel, clientId, resultString,
        connectorName, transPhase;
        bool fl = false;
        for (int i = 0; sprintf(ndx, "%d", i) &&
                (r.find(std::string("FailedTrans") + string(ndx)) != r.end()); i++)
        {
                transId = r[string("FailedTrans") + string(ndx)].c_str();
                transTimeStamp = r[string("TransTimeStamp") + string(ndx)].c_str();
                transModel = r[string("TransModel") + string(ndx)].c_str();
                clientId = r[string("ClientId") + string(ndx)].c_str();
                resultString = r[string("ResultString") + string(ndx)].c_str();
                //Form3->StatusBar1->SimpleText = "Failure reason: " + resultString;
                connectorName = r[string("ConnectorName") + string(ndx)].c_str();
                transPhase = r[string("TransPhase") + string(ndx)].c_str();

                ListItem = PagesDlg->ListView2->Items->Add();
                ListItem->Caption = transId;
                ListItem->SubItems->Add(transTimeStamp);
                ListItem->SubItems->Add(transModel);
                ListItem->SubItems->Add(clientId);
                ListItem->SubItems->Add(resultString);
                ListItem->SubItems->Add(connectorName);
                ListItem->SubItems->Add(transPhase);
                fl = true;
        }
        if (fl && PagesDlg->Button6->Caption == "Stop Engine")
                PagesDlg->ImageList2->GetBitmap(2, PagesDlg->Image2->Picture->Bitmap);
        else if (PagesDlg->Button6->Caption == "Stop Engine")
                PagesDlg->ImageList2->GetBitmap(3, PagesDlg->Image2->Picture->Bitmap);
        else
                PagesDlg->ImageList2->GetBitmap(1, PagesDlg->Image2->Picture->Bitmap);

        PagesDlg->Image2->Refresh();
}

//---------------------------------------------------------------------------
void EngineRequest::UnlockEngine() throw(exception)
{
        //-------------------------------------------------------------------
        // Release the engine console lock
        r.clear();
        r["sysRequest"] = "Un-Lock";

        Request(r);
}

//---------------------------------------------------------------------------
void EngineRequest::GetTransCode(string transName, TMemo *memo) throw(exception)
{
        r.clear();
        r["sysRequest"] = "Get Trans Code";
        r["sysTransName"] = transName;

        Request(r);

        memo->Clear();
        for (int i = 0; sprintf(ndx, "%d", i) &&
                (r.find(string("sysTextLine") + ndx) != r.end()); i++)
                memo->Lines->Add(AnsiString(r[string("sysTextLine") + ndx].c_str()));
                //Form2->Memo1->Lines->Add(AnsiString(r[string("sysTextLine") + ndx].c_str()));
        codeFileName = r["sysFileName"];
}

//---------------------------------------------------------------------------
void EngineRequest::SaveTransCode(TMemo *memo) throw(exception)
{
        if (codeFileName == "")
                throw runtime_error("No file name");
        r.clear();
        r["sysClientId"] = "RuntimeMonitor";
        r["sysRequest"] = "Register Transaction";
        r["sysInstall"] = "True";
        r["sysData"] = memo->Text.c_str();
        r["sysFileName"] = codeFileName;

        Request(r);
}

//---------------------------------------------------------------------------
void EngineRequest::GetConfigValues(string servletName, TAdvStringGrid *strG) throw(exception)
{
        r.clear();
        r["sysRequest"] = "Get Config Values";
        r["sysServletName"] = servletName;
        Request(r);

        strG->FixedRows = 0;
        strG->Cols[0]->Clear();
        strG->Cols[1]->Clear();
        strG->RowCount = 2;

        strG->Cols[0]->Add("Parameter");
        strG->Cols[1]->Add("Value");
        strG->FixedRows = 1;

        char ndx[256];
        AnsiString parameterName, parameterValue;
        for (int i = 0; sprintf(ndx, "%d", i) &&
                r.find(string("sysConfigParamName") + ndx) != r.end(); i++)
        {
                parameterName = r[string("sysConfigParamName") + ndx].c_str();
                parameterValue = r[string("sysConfigParamValue") + ndx].c_str();
                strG->Cols[0]->Add(parameterName);
                strG->Cols[1]->Add(parameterValue);
                strG->RowCount++;
        }
        strG->RowCount--;
        strG->EditorMode = false;

        if (r.find("sysRunStatus") != r.end())
        {
                TListItem *item = PagesDlg->ListView1->Selected;
                item->StateIndex = (r["sysRunStatus"] == "Running") ? 0 : 4;
        }
}

//---------------------------------------------------------------------------
void EngineRequest::GetConfigValues(TListItem *item, TAdvStringGrid *strG) throw(exception)
{
        r.clear();
        r["sysRequest"] = "Get Config Values";
        r["sysServletName"] = item->Caption.c_str();
        //r["sysServletName"] = servletName;

        Request(r);

        strG->FixedRows = 0;
        strG->Cols[0]->Clear();
        strG->Cols[1]->Clear();
        strG->RowCount = 2;

        strG->Cols[0]->Add("Parameter");
        strG->Cols[1]->Add("Value");
        strG->FixedRows = 1;

        char ndx[256];
        AnsiString parameterName, parameterValue;
        for (int i = 0; sprintf(ndx, "%d", i) &&
                r.find(string("sysConfigParamName") + ndx) != r.end(); i++)
        {
                parameterName = r[string("sysConfigParamName") + ndx].c_str();
                parameterValue = r[string("sysConfigParamValue") + ndx].c_str();
                strG->Cols[0]->Add(parameterName);
                strG->Cols[1]->Add(parameterValue);
                strG->RowCount++;
        }
        strG->RowCount--;
        strG->EditorMode = false;

        if (r.find("sysRunStatus") != r.end())
                item->StateIndex = (r["sysRunStatus"] == "Running") ? 0 : 4;
}

//---------------------------------------------------------------------------
void EngineRequest::SetParam(string servletName, string paramName,
                string paramValue) throw(exception)
{
        r.clear();
        r["sysRequest"] = "Set Param";
        r["sysServletName"] = servletName;
        r["sysParamName"] = paramName;
        r["sysParamValue"] = paramValue;

        Request(r);
}

//---------------------------------------------------------------------------
void EngineRequest::GetRecoveryPoints(string transId, string lastPoint) throw(exception)
{
        r.clear();
        r["sysRequest"] = "List Recovery Points";
        r["sysTransId"] = transId;
        r["sysLastPhase"] = lastPoint;

        Request(r);

        for (int i = 0; i < Form3->ComboBox1->Items->Count; i++)
                delete Form3->ComboBox1->Items->Objects[i];

        Form3->ComboBox1->Clear();
        Form3->ComboBox1->Enabled = true;

        AnsiString val;
        for (int i = 0; sprintf(ndx, "%d", i) &&
                (r.find(string("PhaseNumber") + ndx) != r.end()); i++)
        {
                val = AnsiString(r[string("PhaseNumber") + ndx].c_str());
                if ((r[string("FailedServlet") + ndx] == "No Name") && (val != "0"))
                        continue;
                if (r[string("FailedServlet") + ndx] == "No Name")
                        val += " - Start";
                else
                        val += " - " + AnsiString(r[string("FailedServlet") + ndx].c_str());

                RecoveryInfo *ri = new RecoveryInfo();
                ri->phaseNumber = r[string("PhaseNumber") + ndx];
                ri->failedServlet = r[string("FailedServlet") + ndx];
                ri->timeStamp = r[string("TimeStamp") + ndx];
                ri->transId = transId;
                Form3->ComboBox1->Items->AddObject(val, ri);
                Form3->ComboBox1->ItemIndex = 0;
        }
}
//---------------------------------------------------------------------------
void EngineRequest::GetPhaseData(string transId, string phaseNumber) throw(exception)
{
 extern AnsiString lastPoint;

        r.clear();
        r["sysRequest"] = "Get Phase Data";
        r["sysTransId"] = transId;
        r["sysPhaseNumber"] = phaseNumber;

        Request(r);

        Form3->AdvStringGrid1->Options << goEditing;
        Form3->AdvStringGrid1->FixedRows = 0;
        Form3->AdvStringGrid1->Cols[0]->Clear();
        Form3->AdvStringGrid1->Cols[1]->Clear();
        Form3->AdvStringGrid1->RowCount = 2;

        Form3->AdvStringGrid1->Cols[0]->Add("Parameter");
        Form3->AdvStringGrid1->Cols[1]->Add("Value");
        Form3->AdvStringGrid1->FixedRows = 1;

        char ndx[256];
        AnsiString parameterName, parameterValue;
        for (int i = 0; sprintf(ndx, "%d", i) &&
                r.find(string("sysParameter") + ndx) != r.end(); i++)
        {
                parameterName = r[string("sysParameter") + ndx].c_str();
                parameterValue = r[string("sysValue") + ndx].c_str();
                Form3->AdvStringGrid1->Cols[0]->Add(parameterName);
                Form3->AdvStringGrid1->Cols[1]->Add(parameterValue);
                Form3->AdvStringGrid1->RowCount++;
        }
        Form3->AdvStringGrid1->RowCount--;
        Form3->AdvStringGrid1->EditorMode = false;
}

//---------------------------------------------------------------------------
void EngineRequest::GetTransSchema(string transName) throw(exception)
{
        r.clear();
        r["sysRequest"] = "Get Trans Schema";
        r["sysTransName"] = transName;

        Request(r);
}

//---------------------------------------------------------------------------
void EngineRequest::DeleteFailedTransaction(string transId) throw(exception)
{
        r.clear();
        r["sysRequest"] = "Delete Failed Trans";
        r["sysTransId"] = transId;

        Request(r);
        GetFailedTransactions();
}

//---------------------------------------------------------------------------
void EngineRequest::Refresh() throw(exception)
{
        try {
                GetTransactionModels();
                ListConnectors();
                ListTransformers();
                GetFailedTransactions();
                ListBizProc();
        } catch (Exception &exception){
                Application->ShowException(&exception);
        } catch (exception const &msg) {
                ErrorDlg1->Label1->Caption = AnsiString("ERR Refresh " +
                        AnsiString(msg.what()));
                ErrorDlg1->ShowModal();
        }
}

//---------------------------------------------------------------------------
void EngineRequest::StartEngine() throw(exception)
{
        r.clear();
        r["sysRequest"] = "Set Engine State";
        r["sysEngineState"] = "Run";
        Request(r);
        if (r.find("sysResultStatus") != r.end() &&
                r["sysResultStatus"] == "0")
        {
                PagesDlg->Button6->Caption = "Stop Engine";
                PagesDlg->Label3->Caption = (AnsiString("Running since ")
                        + AnsiString(r["sysStartDate"].c_str()));
                PagesDlg->Label4->Caption = AnsiString(r["sysServletCount"].c_str());
                PagesDlg->Label15->Caption =AnsiString(r["sysTransFailed"].c_str());
                PagesDlg->Label16->Caption =AnsiString(r["sysTransCount"].c_str());
                PagesDlg->Label14->Caption =AnsiString(r["sysTransCompleted"].c_str());
                if (PagesDlg->ListView2->Items->Count)
                        PagesDlg->ImageList2->GetBitmap(2, PagesDlg->Image2->Picture->Bitmap);
                else
                        PagesDlg->ImageList2->GetBitmap(3, PagesDlg->Image2->Picture->Bitmap);
                PagesDlg->Image2->Refresh();
        }
}

//---------------------------------------------------------------------------
void EngineRequest::StopEngine() throw(exception)
{
        r.clear();
        r["sysRequest"] = "Set Engine State";
        r["sysEngineState"] = "Halt";
        Request(r);
        if (r.find("sysResultStatus") != r.end() &&
                r["sysResultStatus"] == "0")
        {
                PagesDlg->Button6->Caption = "Start Engine";
                PagesDlg->Label3->Caption = (AnsiString("Halted since ")
                        + AnsiString(r["sysStopDate"].c_str()));
                PagesDlg->Label4->Caption = AnsiString(r["sysServletCount"].c_str());
                PagesDlg->Label15->Caption =AnsiString(r["sysTransFailed"].c_str());
                PagesDlg->Label16->Caption =AnsiString(r["sysTransCount"].c_str());
                PagesDlg->Label14->Caption =AnsiString(r["sysTransCompleted"].c_str());
                PagesDlg->ImageList2->GetBitmap(1, PagesDlg->Image2->Picture->Bitmap);
                PagesDlg->Image2->Refresh();
        }
}

//---------------------------------------------------------------------------
void EngineRequest::SetConsoleParam(NameValueTyp &params) throw(exception)
{
        Request(params);
}

//---------------------------------------------------------------------------
void EngineRequest::ActivateModel(string modelName) throw(exception)
{
        r.clear();
        r["sysRequest"] = "Activate Model";
        r["sysModelName"] = modelName;

        Request(r);
        GetTransactionModels();
}

//---------------------------------------------------------------------------
void EngineRequest::SuspendModel(string modelName) throw(exception)
{
        r.clear();
        r["sysRequest"] = "Suspend Model";
        r["sysModelName"] = modelName;

        Request(r);
        GetTransactionModels();
}
//---------------------------------------------------------------------------
void EngineRequest::StopServlet(string servletName) throw(exception)
{
        r.clear();
        r["sysRequest"] = "Set Run State";
        r["sysState"] = "Halt";
        r["sysServletName"] = servletName;

        Request(r);
}

//---------------------------------------------------------------------------
void EngineRequest::StartServlet(string servletName) throw(exception)
{
        r.clear();
        r["sysRequest"] = "Set Run State";
        r["sysState"] = "Run";
        r["sysServletName"] = servletName;

        Request(r);
}

//---------------------------------------------------------------------------
void EngineRequest::DeleteModel(string modelName) throw(exception)
{
        r.clear();
        r["sysRequest"] = "Delete Model";
        r["sysModelName"] = modelName;

        Request(r);
        GetTransactionModels();
}
//---------------------------------------------------------------------------
void EngineRequest::RunRecovery(string transId, string transPhase) throw(exception)
{
 NameValueTyp params;

        params.clear();
        params["sysRequest"] = "Run Recovery";
        params["sysTransId"] = transId;
        params["sysPhaseNumber"] = transPhase;
        try {
                Request(params);
        } catch (...) {}

        Refresh();
        if (params.find("sysResultStatus") != params.end() &&
                params["sysResultStatus"] == "-1")
        {
                Form3->StatusBar1->SimpleText = "Recovery failed! - " +
                        AnsiString(r["sysResultString"].c_str());
        }
        else
        {
                Form3->StatusBar1->SimpleText = "Recovery succeded, above is what was returned.";
                Form3->AdvStringGrid1->FixedRows = 0;
                Form3->AdvStringGrid1->Cols[0]->Clear();
                Form3->AdvStringGrid1->Cols[1]->Clear();
                Form3->AdvStringGrid1->RowCount = 2;

                Form3->AdvStringGrid1->Cols[0]->Add("Parameter");
                Form3->AdvStringGrid1->Cols[1]->Add("Value");
                Form3->AdvStringGrid1->FixedRows = 1;

                char ndx[256];
                AnsiString parameterName, parameterValue;
                for (int i = 0; sprintf(ndx, "%d", i) &&
                        params.find(string("sysParameter") + ndx) != params.end(); i++)
                {
                        parameterName = params[string("sysParameter") + ndx].c_str();
                        parameterValue = params[string("sysValue") + ndx].c_str();
                        Form3->AdvStringGrid1->Cols[0]->Add(parameterName);
                        Form3->AdvStringGrid1->Cols[1]->Add(parameterValue);
                        Form3->AdvStringGrid1->RowCount++;
                }
                Form3->AdvStringGrid1->RowCount--;
                Form3->AdvStringGrid1->EditorMode = false;
                Form3->ComboBox1->Clear();
                Form3->ComboBox1->Enabled = false;
                Form3->AdvStringGrid1->Options >> goEditing;
        }
}

//---------------------------------------------------------------------------
void EngineRequest::SetPhaseData(string transId, string transPhase, NameValueTyp &params) throw(exception)
{
        r["sysRequest"] = "Set Phase Data";
        r["sysTransId"] = transId;
        r["sysPhaseNumber"] = transPhase;

        int i = 0;
        char ndx[256];
        for (NameValueTyp::iterator it = params.begin(); it != params.end(); ++it, i++)
        {
                sprintf(ndx, "%d", i);
                r[string("ParamName") + ndx] = (*it).first;
                r[string("ParamValue") + ndx] = (*it).second;
        }
        Request(r);
}

//---------------------------------------------------------------------------
void EngineRequest::ListBizProc() throw(exception)
{
        r.clear();
        r["sysRequest"] = "List Biz Procs";

        Request(r);

        char bpNdx[128];
        char mlNdx[128];
        string bpKey;
        TTreeNode *node = NULL;
        //TTreeNode *childNode = NULL;
        PagesDlg->TreeList1->Items->Clear();
        AnsiString bizProcName, bizProcState, modelName, modelState;

        int items = 0;
        for (int i = 0; sprintf(bpNdx, "%d", i) &&
                r.find(string("BizProcName") + bpNdx) != r.end(); i++)
        {
                bpKey = string("BizProcName") + bpNdx;
                bizProcName = r[bpKey].c_str();
                bizProcState = r[string("BizProcState") + bpNdx].c_str();
                PagesDlg->TreeList1->Items->Add(NULL, bizProcName);
                //childNode = PagesDlg->TreeList1->GetNodeAt(i, 0);
                node = PagesDlg->TreeList1->Items->Item[items];
                items++;
                for (int j = 0; sprintf(mlNdx, "%d", j) &&
                        r.find(bpKey + string("ModelName") + mlNdx) != r.end(); j++)
                {
                        modelName = r[bpKey + string("ModelName") + mlNdx].c_str();
                        PagesDlg->TreeList1->Items->AddChild(node, modelName);
                        items++;
                }
        }

/*
        int items = 0;
        for (int i = 0; sprintf(bpNdx, "%d", i) &&
                r.find(string("BizProcName") + bpNdx) != r.end(); i++)
        {
                bpKey = string("BizProcName") + bpNdx;
                bizProcName = r[bpKey].c_str();
                bizProcState = r[string("BizProcState") + bpNdx].c_str();
                PagesDlg->TreeList1->Items->Add(NULL, bizProcName);
                //childNode = PagesDlg->TreeList1->GetNodeAt(i, 0);
                node = PagesDlg->TreeList1->Items->Item[items];
                for (int j = 0; sprintf(mlNdx, "%d", j) &&
                        r.find(bpKey + string("ModelName") + mlNdx) != r.end(); j++)
                {
                        modelName = r[bpKey + string("ModelName") + mlNdx].c_str();
                        PagesDlg->TreeList1->Items->AddChild(node, modelName);
                        items++;
                }
        }
*/
}

