//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "OBJBusinessObject.h"
#include "OBJBusinessObjectField.h"
#include "OBJConnection.h"
#include "OBJConnector.h"
#include "OBJConnectorObject.h"
#include "OBJConnectorObjectField.h"
#include "OBJTextObject.h"

#define TBUSINESSOBJECT_OBJ_VERSION 1


//--------------------------------------------------------------------------- TBUSINESSOBJECT
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - constructor ()
__fastcall TBusinessObject::TBusinessObject() : TConnectableObject()
{
   d_ObjectCode.reset(new TStringList());
   d_Fields.reset(new TBusinessObjectFieldCollection());
   d_Objects.reset(new TStringList());
}


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - constructor (TReader* const)
__fastcall TBusinessObject::TBusinessObject(TReader* const Reader) : TConnectableObject()
{
   d_ObjectCode.reset(new TStringList());
   d_Fields.reset(new TBusinessObjectFieldCollection());
   readData(Reader);
   d_Objects.reset(new TStringList());
}


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - destructor
__fastcall TBusinessObject::~TBusinessObject()
{
   d_Fields->clearObjects();
}


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - readData(TReader* const)
void __fastcall TBusinessObject::readData(TReader* const Reader)
{
   TConnectableObject::readData(Reader);
   Reader->ReadListBegin();
   while (!Reader->EndOfList())
      d_ObjectCode->Add(Reader->ReadString());
   Reader->ReadListEnd();
   d_Fields->readData(Reader);
//   d_Objects->readData(Reader);
}


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - writeData(TWriter* const) const
void __fastcall TBusinessObject::writeData(TWriter* const Writer) const
{
   setVersion(TBUSINESSOBJECT_OBJ_VERSION);
   TConnectableObject::writeData(Writer);
   Writer->WriteListBegin();
   for (int i=0; i<=d_ObjectCode->Count-1; i++)
      Writer->WriteString(d_ObjectCode->Strings[i]);
   Writer->WriteListEnd();
   d_Fields->writeData(Writer);
//   d_Objects->writeData(Writer);
}


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - writeXML(TStream* stream) const
void __fastcall TBusinessObject::writeXML(TStream* stream) const
{
   setVersion(TBUSINESSOBJECT_OBJ_VERSION);
   writeXMLStartTag(stream, "TBusinessObject");
   TConnectableObject::writeXML(stream);
   d_Fields->writeXML(stream);
   writeXMLEndTag(stream, "TBusinessObject");
}


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - showDialog(TComponent* const, bool)
bool __fastcall TBusinessObject::showDialog(TComponent* const Owner, bool ReadOnly)
{
   return false;
}


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - addObjectFields(TConnectorObject* const)
void __fastcall TBusinessObject::addObjectFields(TConnectorObject* const co)
{
   for (TConnectorObjectFieldCollection::CI cofi = co->getFields()->begin(); cofi != co->getFields()->end(); ++cofi)
      d_Fields->addItem(new TBusinessObjectField(co->getParent()->getName()+"."+co->getName()+"."+cofi->second->getName(),
                                                 co->getParent()->getName()+"."+co->getName()+"."+cofi->second->getName()));
}


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - writeScriptCode(TStringList* const)
void __fastcall TBusinessObject::writeScriptCode(TStrings* const code, TStringList* const callcode)
{
   d_EOROA = 255;
   code->AddObject("     create_bo", this);
   code->AddObject("     {", this);

   for (TConnectionCollection::CI cci = d_Connections->begin(); cci != d_Connections->end(); ++cci) {
      TConnectableObject* co = cci->second->getOtherEnd();
      if (co->ClassNameIs("TTextObject"))
         dynamic_cast<TTextObject*>(co)->writeScriptCode(code, 10);

      if (co->ClassNameIs("TConnector") && cci->second->getDirection() == cdOutbound)
         for (int i=0; i<=d_Objects->Count-1; i++) {
            dynamic_cast<TConnectableObject*>(d_Objects->Objects[i])->setEndOfReadOnlyArea(16);
            callcode->AddObject("     run_method "+co->getName()+"."+d_Objects->Strings[i]+".Set"+d_Objects->Strings[i], d_Objects->Objects[i]);
         }
   }

   for (TBusinessObjectFieldCollection::CI bofi = d_Fields->begin(); bofi != d_Fields->end(); ++bofi)
      bofi->second->writeScriptCode(code);

   code->AddObject("     }", this);
   code->Add("");
}


#pragma package(smart_init)
