//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "OBJBooking.h"
#include "OBJItemPrintQueue.h"
#include "OBJReceipt.h"

#include "UIBooking.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)
#pragma link "FR_Class"
#pragma link "FR_DSet"
#pragma link "FR_RRect"

TReceiptPrintQueue* TReceiptPrintQueue::_instance = 0;


__fastcall TItemPrintQueue::TItemPrintQueue() : TObject()
{
   d_Items.reset(new TPersistentExCollection());
}


bool __fastcall TReceiptPrintQueue::showPrintMessage() const
{
   return Application->MessageBox("The queued receipt count will now fill one or more pages.\nDo you wish to print all queued receipts now?",
                                  "Print Queued Receipts?",
                                  MB_YESNO + MB_ICONQUESTION) == IDYES;
}


bool __fastcall TReceiptPrintQueue::printItems() const
{
   // 1) create an FRReport object, parent Application->MainForm
   // 2) create an FRUserDataset Object, parent Application->MainForm, name "UserDataset1"
   // 3) load the report object with receipts.frf
   // 4) set navigation methods for FRUserDatabase, and the getValue method for the FRReport object;
   //    we're traversing getItems()
   // 5) do FRReport->PrintReport()  - I think we want to return a value from that at the
   //    end, so that we can authorise the queue to be cleared.  Any failure, or user decision
   //    to abort the job should keep the queue intact.
   std::auto_ptr<TfrReport> fr(new TfrReport(Application->MainForm));
   fr->LoadFromFile(ExtractFilePath(Application->ExeName) + "Reports\\receipts.frf");
   fr->OnGetValue = getValue;

   std::auto_ptr<TfrUserDataset> fd (new TfrUserDataset(Application->MainForm));
   fd->Name = "frUserDataset1";
   fd->OnCheckEOF = CheckEOF;
   fd->OnFirst = GoFirst;
   fd->OnNext = GoNext;
   fd->OnPrior = GoPrior;

   fr->ShowReport();

   return true;  // for now!
}



void __fastcall TReceiptPrintQueue::getValue(const AnsiString ParName, Variant &ParValue)
{
   if (ParName.UpperCase() == "RECEIPTNBR")
      ParValue = dynamic_cast<TReceipt*>(getCurPtrItem())->getReceiptNbr();

   if (ParName.UpperCase() == "AMOUNT")
      ParValue = FloatToStrF(dynamic_cast<TReceipt*>(getCurPtrItem())->getAmount(), ffFixed, 12, CurrencyDecimals);

   if (ParName.UpperCase() == "BOOKINGNBR")
      ParValue = getCurPtrText().SubString(1, getCurPtrText().Pos(",")-1);

   if (ParName.UpperCase() == "RECEIPTTEXT") {
      TBookingDisplayItem* bkg = BookingMap->find(getCurPtrText().SubString(1, getCurPtrText().Pos(",")-1));
      if (bkg)
         ParValue = dynamic_cast<TReceipt*>(getCurPtrItem())->getDescription() + " for " + bkg->getPassengerNames();
      else
         ParValue = dynamic_cast<TReceipt*>(getCurPtrItem())->getDescription();
   }
}
