//---------------------------------------------------------------------------

#ifndef OBJItemPrintQueueH
#define OBJItemPrintQueueH
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

#define RECEIPTS_PER_PAGE 4

#include "FR_Class.hpp"
#include "FR_DSet.hpp"
#include "FR_RRect.hpp"

#include "OBJPersistentEx.h"
#include "OBJPersistentExList.h"

typedef TPersistentExList<TPersistentEx> TPersistentExCollection;

class TItemPrintQueue : public TObject
{
   public:
      __fastcall TItemPrintQueue();

      virtual __fastcall ~TItemPrintQueue() {};
      void __fastcall addItem(const String& c, TPersistentEx* const pe) {
         d_Items->addItem(c + "," + pe->getCode(), pe);
         d_Start = d_Items->begin();
         d_End = d_Items->end();
         if ( (!(d_Items->size() % getNumberPerPage())) && showPrintMessage() && printItems())
            d_Items->clearObjects();

      }

      void __fastcall addItemAndPrintIt(const String& c, TPersistentEx* const pe) {
         d_Items->addItem(c + "," + pe->getCode(), pe);
         bool emptyQueue = false;
         if (d_Items->size()) {
            if (Application->MessageBox("There are other receipts in the queue.\nDo you want to print them now, too?",
                                        "Print Queued Receipts?", MB_YESNO + MB_ICONINFORMATION) == IDYES) {
               d_Start = d_Items->begin();
               d_End = d_Items->end();
               emptyQueue = true;
            }
            else {
               d_Start = d_Items->getMap().find(pe->getCode());
               d_End = d_Start;
               ++d_End;
            }
         }
         else {
            d_Start = d_Items->getMap().find(pe->getCode());
            d_End = d_Start;
            ++d_End;
         }
         if (printItems())
            if (emptyQueue)
               d_Items->clearObjects();
            else
               d_Items->deleteItem(pe);
      }

      virtual int  __fastcall getNumberPerPage() const = 0;
      virtual bool __fastcall showPrintMessage() const = 0;
      virtual bool __fastcall printItems() const = 0;


      void __fastcall GoFirst(TObject *Sender) {
         d_CurPtr = d_Start;
      }

      void __fastcall GoNext(TObject *Sender)  {
         ++d_CurPtr;
      }

      void __fastcall GoPrior(TObject *Sender) {
         --d_CurPtr;
      }

      void __fastcall CheckEOF(TObject *Sender, bool &Eof) {
         Eof = d_CurPtr == d_End;
      }

      const std::auto_ptr<TPersistentExCollection>& __fastcall getItems() const { return d_Items; }

      TPersistentEx* __fastcall getCurPtrItem() const { return d_CurPtr->second; }
      const String& __fastcall getCurPtrText() const { return d_CurPtr->first; }

   private:
      std::auto_ptr<TPersistentExCollection> d_Items;
      TPersistentExCollection::CI d_CurPtr;
      TPersistentExCollection::CI d_Start;
      TPersistentExCollection::CI d_End;
};


class TReceiptPrintQueue : public TItemPrintQueue
{
   public:
       static TReceiptPrintQueue* Instance() {
          if (!_instance)
             _instance = new TReceiptPrintQueue();
          return _instance;
       }
       __fastcall ~TReceiptPrintQueue() {
          _instance = 0;
       }
       int __fastcall getNumberPerPage() const { return RECEIPTS_PER_PAGE; }
       bool __fastcall showPrintMessage() const;
       bool __fastcall printItems() const;
       void __fastcall getValue(const AnsiString, Variant&);

   protected:
       __fastcall TReceiptPrintQueue() : TItemPrintQueue() { }

   private:
       static TReceiptPrintQueue* _instance;
};



#endif
 