//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "OBJBusinessObject.h"
#include "OBJConditionalObject.h"
#include "OBJConnector.h"
#include "OBJConnectorObject.h"
#include "OBJTextObject.h"
#include "OBJTransaction.h"
#include "OBJTransformer.h"

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#define TTRANSACTION_OBJ_VERSION 1


//---------------------------------------------------------------------------
void __fastcall TTransaction::init()
{
   d_BusinessObjects.reset(new TBusinessObjectCollection());
   d_Connectors.reset(new TConnectorCollection());
   d_TextObjects.reset(new TTextObjectCollection());
   d_ConditionalObjects.reset(new TConditionalObjectCollection());
   d_Transformers.reset(new TTransformerCollection());
}


//---------------------------------------------------------------------------
__fastcall TTransaction::TTransaction() : TConnectableObject()
{
   init();
}


//---------------------------------------------------------------------------
__fastcall TTransaction::TTransaction(TReader* const Reader) : TConnectableObject()
{
   init();
   readData(Reader);
}


//---------------------------------------------------------------------------
__fastcall TTransaction::~TTransaction()
{
   d_BusinessObjects->clearObjects();

   d_Connectors->clearObjects();

   d_TextObjects->clearObjects();

   d_ConditionalObjects->clearObjects();

   d_Transformers->clearObjects();
}

//---------------------------------------------------------------------------
void __fastcall TTransaction::readData(TReader* const Reader)
{
   TConnectableObject::readData(Reader);
   d_BusinessObjects->readData(Reader);
   d_Connectors->readData(Reader);
   d_TextObjects->readData(Reader);
   d_ConditionalObjects->readData(Reader);
   d_Transformers->readData(Reader);
}


//---------------------------------------------------------------------------
void __fastcall TTransaction::writeData(TWriter* const Writer) const
{
   setVersion(TTRANSACTION_OBJ_VERSION);
   TConnectableObject::writeData(Writer);
   d_BusinessObjects->writeData(Writer);
   d_Connectors->writeData(Writer);
   d_TextObjects->writeData(Writer);
   d_ConditionalObjects->writeData(Writer);
   d_Transformers->writeData(Writer);
}


void __fastcall TTransaction::writeXML(TStream* const stream) const
{
   setVersion(TTRANSACTION_OBJ_VERSION);
   writeXMLStartTag(stream, "TTransaction");
   TConnectableObject::writeXML(stream);
   writeXMLEndTag(stream, "TTransaction");
   d_BusinessObjects->writeXML(stream);
   d_Connectors->writeXML(stream);
   d_TextObjects->writeXML(stream);
   d_ConditionalObjects->writeXML(stream);
   d_Transformers->writeXML(stream);
}



//---------------------------------------------------------------------------
bool __fastcall TTransaction::showDialog(TComponent* const Owner, bool ReadOnly)
{
   return false;
}


void __fastcall TTransaction::writeScriptCode(TStrings* const code)
{
   code->AddObject(d_Name + ":", this);
   code->Add("{");
   TStringList* callstrs = new TStringList();

   for (TBusinessObjectCollection::CI boci = d_BusinessObjects->begin(); boci != d_BusinessObjects->end(); ++boci)
      boci->second->writeScriptCode(code, callstrs);

   for (TConditionalObjectCollection::CI coci = d_ConditionalObjects->begin(); coci != d_ConditionalObjects->end(); ++coci)
      coci->second->writeScriptCode(code);

   code->AddStrings(callstrs);
   code->Add("}");
}

void __fastcall TTransaction::updateObject(const String& s)
{
   String str = s;
   if ( s.Pos(":") )
      str = s.SubString(1, s.Pos(":")-1);

   d_Name = str;
}


#pragma package(smart_init)
