//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "dcmemopt.hpp"
#include "dcenvopt.hpp"

#include "OBJConnector.h"
#include "OBJConnectorObject.h"
#include "OBJConnectorObjectField.h"
#include "OBJServletMethod.h"
#include "OBJServletParam.h"
#include "OBJSocket.h"
#include "OBJTransaction.h"
#include "OBJTransformer.h"

#include "UIAbout.h"
#include "UICommandScriptViewer.h"
#include "UIDesignerOptions.h"
#include "UIMain.h"
#include "UISetupComms.h"
#include "UITransactionEditor.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "htmlhint"
#pragma link "Placemnt"
#pragma link "dcmemo"
#pragma link "dcmemopt"

#pragma resource "*.dfm"
TfrmMain *frmMain;
//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
        : TForm(Owner)
{
   Application->ShowHint = true;
   d_te = NULL;
}


//---------------------------------------------------------------------------
void __fastcall TfrmMain::About1Click(TObject *Sender)
{
   std::auto_ptr<TfrmAbout> frmAbout(new TfrmAbout(this));
   frmAbout->Timer1->Enabled = true;
   frmAbout->ShowModal();
}


//---------------------------------------------------------------------------
void __fastcall TfrmMain::aNewTransactionExecute(TObject *Sender)
{
   d_te = TfrmTransactionEditor::Instance(this);

   TTransaction* trans = new TTransaction();
   SendMessage(d_te->Handle, smAddTransaction, 0, (long)trans);
   d_te->Show();
}


//---------------------------------------------------------------------------
void __fastcall TfrmMain::aOpenTransactionExecute(TObject *Sender)
{
   if (OpenDialog1->Execute()) {
      SaveDialog1->FileName = OpenDialog1->FileName;
      TFileStream* stream = new TFileStream(OpenDialog1->FileName, fmOpenRead);
      TReader* Reader = new TReader(stream, 0x5000);
      TTransaction* trans = new TTransaction();
      trans->readData(Reader);
      delete Reader;
      delete stream;
      d_te = TfrmTransactionEditor::Instance(this);
      SendMessage(d_te->Handle, smAddTransaction, 0, (long)trans);
      d_te->Show();
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::Exit1Click(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------


void __fastcall TfrmMain::FormCreate(TObject *Sender)
{
   Constraints->MinHeight = ToolBar1->Height + GetSystemMetrics(SM_CYMENU) + GetSystemMetrics(SM_CYCAPTION) + (GetSystemMetrics(SM_CYDLGFRAME) * 2) + (GetSystemMetrics(SM_CYBORDER) * 2);
   Constraints->MaxHeight = Constraints->MinHeight;
   FormStorage1->RestoreFormPlacement();
   ConnectorCollection = new TConnectorCollection();
   while (ClientSocket1->Host == "" && ClientSocket1->Address == "")
      aSetupCommsExecute(this);
//      ActionList1->ExecuteAction(aSetupComms);
   TransformerCollection = new TTransformerCollection();
   std::auto_ptr<TfrmAbout> frmAbout(new TfrmAbout(this));
   frmAbout->btnClose->Visible = false;
   frmAbout->Animator1->Visible = true;
   frmAbout->lblLoad->Visible = true;
   frmAbout->AnimatedImage1->GlyphNum = 1;
   frmAbout->Show();
   Application->ProcessMessages();

   SocketObj s(ClientSocket1);

   Application->ProcessMessages();

   try {
      s.Open();
   }
   catch (...) {
      frmAbout.reset(0);
      Application->MessageBox("Unable to connect to server.\nEnter correct connection settings in Comms Setup.", "Unable to Connect", MB_OK);
      aNewTransaction->Enabled = false;
      aOpenTransaction->Enabled = false;
      return;
   }

   aNewTransaction->Enabled = true;
   aOpenTransaction->Enabled = true;

   Application->ProcessMessages();

   NameValueTyp r;
   r["sysClientId"] = "TransDesigner";
   r["sysRequest"] = "List Connectors";
   r["sysReply"] = "True";
   r["sysDebug"] = "True";



   s << r;
   r.clear();
   s >> r;
   s.Close();


   for (NameValueTyp::iterator it = r.begin(); it != r.end(); ++it) {
      if (String(it->first.c_str()).Pos("Connector")) {
         TConnector* conn = new TConnector(String(it->second.c_str()), String(it->second.c_str()));
         ConnectorCollection->addItem(conn);

         NameValueTyp c;
         c["sysClientId"] = "TransDesigner";
         c["sysRequest"] = "Get Servlet Spec";
         c["sysConnectorName"] = it->second.c_str();
         c["sysDebug"] = "True";
         c["sysReply"] = "True";
         s.Open();
         s << c;
         c.clear();
         s >> c;
         TStringList* fieldlist = new TStringList();
         TStringList* methodlist = new TStringList();
         std::map<String, String> paramlist;
         for (NameValueTyp::iterator cinfo = c.begin(); cinfo != c.end(); ++cinfo) {
            String temp = cinfo->first.c_str();
            if (String(cinfo->first.c_str()).Pos("Field"))
               fieldlist->Add(cinfo->second.c_str());
            else
               if (String(cinfo->first.c_str()).Pos("Method"))
                  methodlist->Add(cinfo->second.c_str());
               else
                  if (String(cinfo->first.c_str()).Trim().Length() && String(cinfo->first.c_str()).Trim().Length() &&
                      String(cinfo->first.c_str()) != "sysClientId" &&
                      String(cinfo->first.c_str()) != "sysRequest" &&
                      String(cinfo->first.c_str()) != "sysConnectorName" &&
                      String(cinfo->first.c_str()) != "sysDebug" &&
                      String(cinfo->first.c_str()) != "sysReply" &&
                      String(cinfo->first.c_str()) != "sysResultStatus" &&
                      String(cinfo->first.c_str()) != "sysParamName")
                     paramlist[cinfo->first.c_str()] = cinfo->second.c_str();
         }

         for (int i=0; i<=fieldlist->Count-1; i++) {
            String FieldItem = fieldlist->Strings[i];
            FieldItem.Delete(1, FieldItem.Pos("."));

            String ObjName = FieldItem.SubString(1, FieldItem.Pos(".")-1);
            TConnectorObject* connobj = conn->getObjects()->find(ObjName);
            if (!connobj) {
               connobj = new TConnectorObject(conn, ObjName, ObjName);
               conn->getObjects()->addItem(connobj);
            }

            String fieldname = FieldItem.SubString(FieldItem.Pos(".")+1, FieldItem.Length());
            connobj->getFields()->addItem(new TConnectorObjectField(fieldname, fieldname));
            Application->ProcessMessages();
         }

         for (int i=0; i<=methodlist->Count-1; i++) {
            String MethodItem = methodlist->Strings[i];
            MethodItem.Delete(1, MethodItem.Pos("."));

            String ObjName = MethodItem.SubString(1, MethodItem.Pos(".")-1);
            TConnectorObject* connobj = conn->getObjects()->find(ObjName);
            if (!connobj) {
               connobj = new TConnectorObject(conn, ObjName, ObjName);
               conn->getObjects()->addItem(connobj);
            }

            String methodname = MethodItem.SubString(MethodItem.Pos(".")+1, MethodItem.Length());
            connobj->getMethods()->addItem(new TServletMethod(methodname, methodname));
            Application->ProcessMessages();
         }

         for (std::map<String, String>::iterator pi = paramlist.begin(); pi != paramlist.end(); ++ pi) {
            TServletParam* cp = new TServletParam(pi->first, pi->second);
            conn->getParams()->addItem(cp);
         }

         delete fieldlist;
         delete methodlist;
         s.Close();
      }
      Application->ProcessMessages();
   }

   r.clear();

   r["sysClientId"] = "TransDesigner";
   r["sysRequest"] = "List Transformers";
   r["sysReply"] = "True";
   r["sysDebug"] = "True";
   s.Open();
   s << r;
   r.clear();
   s >> r;
   s.Close();

   for (NameValueTyp::iterator it = r.begin(); it != r.end(); ++it) {
      if (String(it->first.c_str()).Pos("Transformer")) {
         TTransformer* transformer = new TTransformer(String(it->second.c_str()), String(it->second.c_str()));
         TransformerCollection->addItem(transformer);

         NameValueTyp c;
         c["sysClientId"] = "TransDesigner";
         c["sysRequest"] = "Get Servlet Spec";
         c["sysConnectorName"] = it->second.c_str();
         c["sysDebug"] = "True";
         c["sysReply"] = "True";
         s.Open();
         s << c;
         c.clear();
         s >> c;
//         TStringList* fieldlist = new TStringList();
         TStringList* methodlist = new TStringList();
         std::map<String, String> paramlist;
         for (NameValueTyp::iterator cinfo = c.begin(); cinfo != c.end(); ++cinfo) {
            String temp = cinfo->first.c_str();
//            if (String(cinfo->first.c_str()).Pos("Field"))
//               fieldlist->Add(cinfo->second.c_str());
//            else
               if (String(cinfo->first.c_str()).Pos("Method"))
                  methodlist->Add(cinfo->second.c_str());
               else
                  if (String(cinfo->first.c_str()).Pos("sysConfigParamName") ||
                      String(cinfo->first.c_str()).Pos("sysParam"))
                     paramlist[cinfo->first.c_str()] = cinfo->second.c_str();
         }

         for (int i=0; i<=methodlist->Count-1; i++) {
            String MethodItem = methodlist->Strings[i];
            MethodItem.Delete(1, MethodItem.Pos("."));

//            String methodname = MethodItem.SubString(MethodItem.Pos(".")+1, MethodItem.Length());
            transformer->getMethods()->addItem(new TServletMethod(MethodItem, MethodItem));
            Application->ProcessMessages();
         }

         for (std::map<String, String>::iterator pi = paramlist.begin(); pi != paramlist.end(); ++ pi) {
            TServletParam* cp = new TServletParam(pi->first, pi->second);
            transformer->getParams()->addItem(cp);
         }

//         delete fieldlist;
         delete methodlist;
         s.Close();
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::ClientSocket1Error(TObject *Sender,
      TCustomWinSocket *Socket, TErrorEvent ErrorEvent, int &ErrorCode)
{
   Application->MessageBox(String("An Error Occured: " + IntToStr(ErrorCode)).c_str(), "DEBUG", MB_OK);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::aSetupCommsExecute(TObject *Sender)
{
   std::auto_ptr<TfrmSetupComms> sc(new TfrmSetupComms(this));
   if (ClientSocket1->Host.Length())
      sc->ebHost->Text = ClientSocket1->Host;
   else
      sc->ebIP->IPString = ClientSocket1->Address;
   sc->ebPort->Text = ClientSocket1->Port;
   if (sc->ShowModal() == mrOk) {
      if (sc->ebHost->Text.Length()) {
         ClientSocket1->Host = sc->ebHost->Text;
         ClientSocket1->Address = "";
      }
      else {
         ClientSocket1->Address = sc->ebIP->IPString;
         ClientSocket1->Host = "";
      }
      ClientSocket1->Port = StrToInt(sc->ebPort->Text);
      ConnectorCollection->clearObjects();
      delete ConnectorCollection;
      if (Visible)
         FormStorage1->SaveFormPlacement();
      FormCreate(Sender);
   }
}
//---------------------------------------------------------------------------





void __fastcall TfrmMain::aDesignEnvironmentOptionsExecute(TObject *Sender)
{
   if (d_te) {
      frmDesignerOptions = new TfrmDesignerOptions(d_te);
      frmDesignerOptions->ColorComboBox1->ColorValue = d_te->ScrollBox1->Color;
      frmDesignerOptions->OriginalColor = d_te->ScrollBox1->Color;
      frmDesignerOptions->Show();
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::aScriptEnvironmentOptionsExecute(TObject *Sender)
{
   ShowEnvironmentOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::Save1Click(TObject *Sender)
{
   if (SaveDialog1->Execute()) {
      TFileStream* stream = new TFileStream(SaveDialog1->FileName, fmCreate);
      TWriter* Writer = new TWriter(stream, 0x5000);
      dynamic_cast<TTransaction*>(d_te->TabControl1->Tabs->Objects[d_te->TabControl1->TabIndex])->writeData(Writer);
      delete Writer;
      delete stream;
   }
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


