//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "OBJBusinessObject.h"
#include "OBJBusinessObjectField.h"
#include "OBJBusinessObjectFieldDragObject.h"
#include "OBJConditionalObject.h"
#include "OBJConnection.h"
#include "OBJConnector.h"
#include "OBJConnectorObject.h"
#include "OBJConnectorObjectField.h"
#include "OBJServletMethod.h"
#include "OBJConnectorObjectDragObject.h"
#include "OBJConnectorObjectFieldDragObject.h"
#include "OBJServletMethodDragObject.h"
#include "OBJServletParam.h"
#include "OBJSocket.h"
#include "OBJTextObject.h"
#include "OBJTransformer.h"
#include "OBJTransaction.h"

#include "UIAnimateLoad.h"
#include "UIMain.h"
#include "UICommandScriptViewer.h"
#include "UIConnectionManager.h"
#include "UIConnectorObjects.h"
#include "UIObjectInspector.h"
#include "UIPersistenceRulesEditor.h"
#include "UIStatementEditor.h"
#include "UITransactionEditor.h"
#include "rmBtnCombo.hpp"
#include "rmInspectorItems.hpp"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "rmInspector"
#pragma link "RegularExpression"
#pragma link "RegularExpressionCustom"                         
#pragma link "RegularExpressionCustom"
#pragma link "ToolEdit"
#pragma resource "*.dfm"

#include "OBJConnectableObject.h"

#define SEP_CHAR '^'

TfrmObjectInspector *frmObjectInspector;
//---------------------------------------------------------------------------
__fastcall TfrmObjectInspector::TfrmObjectInspector(TComponent* Owner, TConnectableObject* co)
        : TForm(Owner)
{
   d_co = co;
   Caption = "Inspecting " + d_co->getName();
   rebuild(co);
}

void __fastcall TfrmObjectInspector::rebuild(TConnectableObject* co)
{
   rmInspector1->ItemIndex = -1;
   Application->ProcessMessages();
   LockWindowUpdate(rmInspector1->Handle);
   rmInspector1->Items->Clear();
   rmInspector1->SepChar = SEP_CHAR;
   rmInspector1->AddInspectorItem("Connections", "<Connections>", __classid(TrmComplexInspectorItem));
   if (co->ClassType() != __classid(TConnector)  && co->ClassType() != __classid(TTransformer))
      /*TrmStringInspectorItem* is = */dynamic_cast<TrmStringInspectorItem*>(rmInspector1->AddInspectorItem("Object Name", co->getName(), __classid(TrmStringInspectorItem)));

   if (co->ClassType() == __classid(TTextObject))
     rmInspector1->AddInspectorItem("Text", dynamic_cast<TTextObject*>(co)->getText()->Text, __classid(TrmComplexInspectorItem));

   if (co->ClassType() == __classid(TBusinessObject)) {
     rmInspector1->AddInspectorItem("Fields", "<Fields>", __classid(TrmComplexInspectorItem));
     TBusinessObject* bo = dynamic_cast<TBusinessObject*>(co);
     for (TBusinessObjectFieldCollection::CI bofi = bo->getFields()->begin(); bofi != bo->getFields()->end(); ++bofi) {
        TrmStringInspectorItem* ii = dynamic_cast<TrmStringInspectorItem*>(rmInspector1->AddInspectorItem("Fields/"+bofi->second->getName(), bofi->second->getMapping(), __classid(TrmStringInspectorItem)));
        if (bofi->second->getMapping().Length())
           ii->ImageIndex = 0;
        else
           ii->ImageIndex = 1;
     }

     rmInspector1->AddInspectorItem("Object Code", "<Object Code>", __classid(TrmComplexInspectorItem));
     rmInspector1->AddInspectorItem("Persistence Rules", "<Persistence Rules>", __classid(TrmComplexInspectorItem));
   }

   if (co->ClassType() == __classid(TConnector)) {
      TConnector* conn = dynamic_cast<TConnector*>(co);
      std::auto_ptr<TfrmAnimateLoad> al(new TfrmAnimateLoad(this));
      al->Show();
      Application->ProcessMessages();
      TrmComboInspectorItem* cii = dynamic_cast<TrmComboInspectorItem*>(rmInspector1->AddInspectorItem("Connector", co->getName(), __classid(TrmComboInspectorItem)));
      for (TConnectorCollection::CI ci = ConnectorCollection->begin(); ci != ConnectorCollection->end(); ++ci)
         cii->Items->Add(ci->second->getName());
      cii->Value = co->getName();


      NameValueTyp r;
      r["sysClientId"] = "BusinessProcessModeller";
      r["sysRequest"] = "Get Config Values";
      r["sysConnectorName"] = co->getName().c_str();
      r["sysReply"] = "True";
      r["sysDebug"] = "True";

      SocketObj s(frmMain->ClientSocket1);
      try {
         s.Open();
         s << r;
         r.clear();
         s >> r;
         s.Close();
      }
      catch (...) {

      }


      for (TServletParamCollection::CI cpi = conn->getParams()->begin(); cpi != conn->getParams()->end(); ++cpi) {
         if (cpi->second->getName().Trim().Length()) {
            if (cpi->first.Pos("sysConfigParamName")) {
               String paramNbr = cpi->first.SubString(19, 4);
               String ParamAllow = "sysParamAllow" + paramNbr;
               String ParamDisplay = "sysParamDisplay" + paramNbr;
               String ParamSize = "sysParamSize" + paramNbr;

               NameValueTyp::iterator val = r.find(String("sysConfigParamValue"+paramNbr).c_str());
               String valuestr = "";
               if (val != r.end())
                  valuestr = String(val->second.c_str());

               TrmStringInspectorItem* ii = dynamic_cast<TrmStringInspectorItem*>(rmInspector1->AddInspectorItem(cpi->second->getName(), valuestr, __classid(TrmStringInspectorItem)));
               ii->UserBool = true;
               TServletParam* pDisplay = conn->getParams()->find(ParamDisplay);
               if (pDisplay && pDisplay->getName() == "False")
                  ii->PasswordChar = '*';
               TServletParam* pSize = conn->getParams()->find(ParamSize);
               if (pSize)
                  ii->MaxLength = StrToIntDef(pSize->getName(), 255);
               TServletParam* pAllow = conn->getParams()->find(ParamAllow);
               if (pAllow)
                  ii->RegExpression = pAllow->getName();
            }
         }
      }

      rmInspector1->AddInspectorItem("Data Model", "<Data Model>", __classid(TrmComplexInspectorItem));
      rmInspector1->AddInspectorItem("Objects", "<Connector Objects>", __classid(TrmComplexInspectorItem));
      for (TConnectorObjectCollection::CI coci = conn->getObjects()->begin(); coci != conn->getObjects()->end(); ++coci) {
         rmInspector1->AddInspectorItem("Objects" + String(SEP_CHAR) + coci->second->getName(), "<"+coci->second->getName()+">", __classid(TrmComplexInspectorItem));
         for (TConnectorObjectFieldCollection::CI cofi = coci->second->getFields()->begin(); cofi != coci->second->getFields()->end(); ++cofi) {
            TrmCustomInspectorItem* cii = rmInspector1->AddInspectorItem("Objects" + String(SEP_CHAR) +coci->second->getName()+String(SEP_CHAR)+cofi->second->getName(), "<"+cofi->second->getName()+">", __classid(TrmComplexInspectorItem));
            cii->ImageIndex = 2;
         }


// Methods don't appear in the object inspector anymore... 15/08/2001.
// .... Yes, they do!  16/08/2001
         for (TServletMethodCollection::CI comi = coci->second->getMethods()->begin(); comi != coci->second->getMethods()->end(); ++comi) {
            TrmCustomInspectorItem* cii = rmInspector1->AddInspectorItem("Objects"+String(SEP_CHAR)+coci->second->getName()+"" + comi->second->getName(), "<"+comi->second->getName()+">", __classid(TrmComplexInspectorItem));
            cii->ImageIndex = 3;
         }
      }
   }

   if (co->ClassType() == __classid(TTransformer)) {
      Application->ProcessMessages();
      TTransformer* tr = dynamic_cast<TTransformer*>(co);
      TrmComboInspectorItem* cii = dynamic_cast<TrmComboInspectorItem*>(rmInspector1->AddInspectorItem("Transformer", co->getName(), __classid(TrmComboInspectorItem)));
      for (TTransformerCollection::CI ci = TransformerCollection->begin(); ci != TransformerCollection->end(); ++ci)
         cii->Items->Add(ci->second->getName());
      cii->Value = co->getName();

      NameValueTyp r;
      r["sysClientId"] = "BusinessProcessModeller";
      r["sysRequest"] = "Get Config Values";
      r["sysConnectorName"] = tr->getName().c_str();
      r["sysReply"] = "True";
      r["sysDebug"] = "True";

      SocketObj s(frmMain->ClientSocket1);
      try {
         s.Open();
         s << r;
         r.clear();
         s >> r;
         s.Close();
      }
      catch (...) {

      }

      for (TServletParamCollection::CI cpi = tr->getParams()->begin(); cpi != tr->getParams()->end(); ++cpi) {
         if (cpi->second->getName().Trim().Length()) {
            if (cpi->first.Pos("sysConfigParamName")) {
               String paramNbr = cpi->first.SubString(19, 4);
               String ParamAllow = "sysParamAllow" + paramNbr;
               String ParamDisplay = "sysParamDisplay" + paramNbr;
               String ParamSize = "sysParamSize" + paramNbr;

               NameValueTyp::iterator val = r.find(String("sysConfigParamValue"+paramNbr).c_str());
               String valuestr = "";
               if (val != r.end())
                  valuestr = String(val->second.c_str());

               TrmStringInspectorItem* ii = dynamic_cast<TrmStringInspectorItem*>(rmInspector1->AddInspectorItem(cpi->second->getName(), valuestr, __classid(TrmStringInspectorItem)));
               ii->UserBool = true;
               TServletParam* pDisplay = tr->getParams()->find(ParamDisplay);
               if (pDisplay && pDisplay->getName() == "False")
                  ii->PasswordChar = '*';
               TServletParam* pSize = tr->getParams()->find(ParamSize);
               if (pSize)
                  ii->MaxLength = StrToIntDef(pSize->getName(), 255);
               TServletParam* pAllow = tr->getParams()->find(ParamAllow);
               if (pAllow)
                  ii->RegExpression = pAllow->getName();
            }
         }
      }
//      if (ci->second->getName().Pos("Field")
//         rmInspector1->AddInspectorItem("Fields", "<Fields>", __classid(TrmComplexInspectorItem));
//      if (ci->second->getName().Pos("Method")
      rmInspector1->AddInspectorItem("Methods", "<Methods>", __classid(TrmComplexInspectorItem));
      for (TServletMethodCollection::CI comi = tr->getMethods()->begin(); comi != tr->getMethods()->end(); ++comi) {
         TrmCustomInspectorItem* cii = rmInspector1->AddInspectorItem("Methods"+String(SEP_CHAR)+comi->second->getName(), "<"+comi->second->getName()+">", __classid(TrmComplexInspectorItem));
         cii->ImageIndex = 3;
      }
   }

   if (co->ClassType() == __classid(TConditionalObject)) {
      rmInspector1->AddInspectorItem("Statement Code", "<Statement Code>", __classid(TrmComplexInspectorItem));
   }
   LockWindowUpdate(0);
   rmInspector2->Items->Clear();
   rmInspector2->AddInspectorItem("Height", co->getGUIHeight(), __classid(TrmIntegerInspectorItem));
   rmInspector2->AddInspectorItem("Left", co->getGUILeft(), __classid(TrmIntegerInspectorItem));
   rmInspector2->AddInspectorItem("Top", co->getGUITop(), __classid(TrmIntegerInspectorItem));
   rmInspector2->AddInspectorItem("Width", co->getGUIWidth(), __classid(TrmIntegerInspectorItem));
   rmInspector1->OnMouseDown = rmInspector1MouseDown;

}

void __fastcall TfrmObjectInspector::SMRefreshInspectorHandler(TMessage& msg)
{
   rebuild(d_co);
}

//---------------------------------------------------------------------------
void __fastcall TfrmObjectInspector::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   TfrmTransactionEditor* te = TfrmTransactionEditor::Instance(this);
   SendMessage(te->Handle, smInspectorClosed, 0, (long)this);
   Action = caFree;
}
//---------------------------------------------------------------------------



void __fastcall TfrmObjectInspector::rmInspector1ComplexEdit(
      TObject *Sender)
{
   if (rmInspector1->CurrentItemPath.Pos("Connections")) {
      // show connection editor
      TfrmConnectionManager* cm = new TfrmConnectionManager(Owner, d_co);
      cm->Show();
   }

//   if (rmInspector1->CurrentItemPath.Pos("Objects")) {
//      TfrmConnectorObjects* fco = new TfrmConnectorObjects(Owner, d_co);
//      fco->Show();
//   }


   if (rmInspector1->CurrentItemPath.Pos("Statement Code")) {
      TfrmStatementEditor* se = new TfrmStatementEditor(Owner, dynamic_cast<TConditionalObject*>(d_co));
      se->Show();
   }

   if (rmInspector1->CurrentItemPath.Pos("Persistence Rules")) {
      TfrmPersistenceRulesEditor* pre = new TfrmPersistenceRulesEditor(Owner, d_co);
      pre->Show();
   }

//   if (rmInspector1->CurrentItemPath.Pos("Object Fields")) {
//      TfrmBusinessObjectFieldsEditor* bofe = new TfrmBusinessObjectFieldsEditor(Owner, d_co);
//      bofe->Show();
//   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmObjectInspector::rmInspector1EditorKeyPress(
      TObject *Sender, char &Key)
{
   if (rmInspector1->CurrentItemPath.Pos("Object Name")) {
      String validChars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_\b";
      if (!validChars.Pos(String(Key).UpperCase()))
         Key = 0;
   }
   if ((d_co->ClassNameIs("TConnector") || d_co->ClassNameIs("TTransformer")) && Key != '\b') {
      TrmStringInspectorItem* ii = dynamic_cast<TrmStringInspectorItem*>(rmInspector1->CurrentItem);
      if (ii->RegExpression.Length()) {
         String x = ii->RegExpression;
         RegularExpression1->Expression = ii->RegExpression;
         if (RegularExpression1->ParseExpression() == RegularExpression::REG_NOERROR)  {
            String m_str = d_EditorString;
            m_str += String(Key);
            RegularExpression1->StringToMatch = m_str;
            if (!RegularExpression1->Match())
               Key = 0;
            else
               d_EditorString += String(Key);
         }
      }
   }
   if (d_co->ClassNameIs("TBusinessObject") && Key == '\"')
      Key = 0;
}
//---------------------------------------------------------------------------



void __fastcall TfrmObjectInspector::rmInspector1EditorCreated(
      TObject *Sender)
{
   if (String(rmInspector1->Editor->ClassName()) == "TrmInspectorEdit") {
      dynamic_cast<TrmCustomEdit*>(rmInspector1->Editor)->OnKeyPress = rmInspector1EditorKeyPress;
      dynamic_cast<TrmCustomEdit*>(rmInspector1->Editor)->OnKeyUp = rmInspector1EditorKeyUp;
      dynamic_cast<TrmCustomEdit*>(rmInspector1->Editor)->OnDragOver = rmInspector1DragOver;
      dynamic_cast<TrmCustomEdit*>(rmInspector1->Editor)->OnDragDrop = rmInspector1DragDrop;
//      dynamic_cast<TrmCustomEdit*>(rmInspector1->Editor)->OnExit = rmInspector1EditorExit;
//      rmInspector1->CurrentItem->GetValueFromEditor(rmInspector1->Editor);
      d_EditorString = dynamic_cast<TrmCustomEdit*>(rmInspector1->Editor)->Text;
   }
   if (String(rmInspector1->Editor->ClassName()) == "TrmInspectorBtnCombo")
      dynamic_cast<TrmBtnCombo*>(rmInspector1->Editor)->OnChange = rmInspector1ComboChanged;
//   else Application->MessageBox(String(rmInspector1->Editor->ClassName()).c_str(), "DEBUG", MB_OK);

}
//---------------------------------------------------------------------------

void __fastcall TfrmObjectInspector::rmInspector1ComboChanged(TObject* Sender)
{
   if (rmInspector1->CurrentItemPath.Pos("Connector")) {
      rmInspector1->CurrentItem->GetValueFromEditor(rmInspector1->Editor);
      TConnector* cnn = ConnectorCollection->findName(rmInspector1->CurrentItem->AsString);
      if (cnn) {
         dynamic_cast<TConnector*>(d_co)->setName(cnn->getName());
         dynamic_cast<TConnector*>(d_co)->setCode(cnn->getCode());

         dynamic_cast<TConnector*>(d_co)->getObjects()->clearObjects();
         dynamic_cast<TConnector*>(d_co)->getParams()->clearObjects();

         for (TConnectorObjectCollection::CI coi = cnn->getObjects()->begin(); coi != cnn->getObjects()->end(); ++coi) {
            TConnectorObject* newconn = new TConnectorObject(dynamic_cast<TConnector*>(d_co), coi->first, coi->first);
            dynamic_cast<TConnector*>(d_co)->getObjects()->addItem(newconn);

            for (TConnectorObjectFieldCollection::CI cofi = coi->second->getFields()->begin(); cofi != coi->second->getFields()->end(); ++cofi)
               newconn->getFields()->addItem(new TConnectorObjectField(cofi->first, cofi->first));
            for (TServletMethodCollection::CI comi = coi->second->getMethods()->begin(); comi != coi->second->getMethods()->end(); ++comi)
               newconn->getMethods()->addItem(new TServletMethod(comi->first, comi->first));
         }
         for (TServletParamCollection::CI cpi = cnn->getParams()->begin(); cpi != cnn->getParams()->end(); ++cpi) {
            TServletParam* newparam = new TServletParam(cpi->second->getCode(), cpi->second->getName());
            dynamic_cast<TConnector*>(d_co)->getParams()->addItem(newparam);
         }
         rmInspector1->OnMouseDown = NULL;
         TNAMEINFO* name_info = new TNAMEINFO;
         name_info->obj = d_co;
         name_info->name =  d_co->getName();
         SendMessage(dynamic_cast<TForm*>(Owner)->Handle, smObjectNameChanged, 0, (long)name_info);
         PostMessage(Handle, smRefreshInspector, 0 , 0);
      }
   }

   if (rmInspector1->CurrentItemPath.Pos("Transformer")) {
      rmInspector1->CurrentItem->GetValueFromEditor(rmInspector1->Editor);
      TTransformer* cnn = TransformerCollection->findName(rmInspector1->CurrentItem->AsString);
      if (cnn) {
         dynamic_cast<TTransformer*>(d_co)->setName(cnn->getName());
         dynamic_cast<TTransformer*>(d_co)->setCode(cnn->getCode());

//         dynamic_cast<TTransformer*>(d_co)->getObjects()->clearObjects();
//         dynamic_cast<TTransformer*>(d_co)->getParams()->clearObjects();

//         for (TTransformerObjectCollection::CI coi = cnn->getObjects()->begin(); coi != cnn->getObjects()->end(); ++coi) {
//            TTransformerObject* newconn = new TTransformerObject(dynamic_cast<TTransformer*>(d_co), coi->first, coi->first);
//            dynamic_cast<TTransformer*>(d_co)->getObjects()->addItem(newconn);
//
//            for (TTransformerObjectFieldCollection::CI cofi = coi->second->getFields()->begin(); cofi != coi->second->getFields()->end(); ++cofi)
//               newconn->getFields()->addItem(new TTransformerObjectField(cofi->first, cofi->first));
//            for (TTransformerObjectMethodCollection::CI comi = coi->second->getMethods()->begin(); comi != coi->second->getMethods()->end(); ++comi)
//               newconn->getMethods()->addItem(new TTransformerObjectMethod(comi->first, comi->first));
//         }
//         for (TTransformerParamCollection::CI cpi = cnn->getParams()->begin(); cpi != cnn->getParams()->end(); ++cpi) {
//            TTransformerParam* newparam = new TTransformerParam(cpi->second->getCode(), cpi->second->getName());
//            dynamic_cast<TTransformer*>(d_co)->getParams()->addItem(newparam);
//         }
         rmInspector1->OnMouseDown = NULL;
         TNAMEINFO* name_info = new TNAMEINFO;
         name_info->obj = d_co;
         name_info->name =  d_co->getName();
         SendMessage(dynamic_cast<TForm*>(Owner)->Handle, smObjectNameChanged, 0, (long)name_info);
         PostMessage(Handle, smRefreshInspector, 0 , 0);
      }
   }


}





void __fastcall TfrmObjectInspector::rmInspector1EditorKeyUp(
      TObject *Sender, WORD &Key, TShiftState Shift)
{
   if (rmInspector1->CurrentItemPath.Pos("Object Name")) {
      TNAMEINFO* name_info = new TNAMEINFO;
      name_info->obj = d_co;
      rmInspector1->CurrentItem->GetValueFromEditor(rmInspector1->Editor);
      name_info->name =  dynamic_cast<TrmStringInspectorItem*>(rmInspector1->CurrentItem)->Value;
      SendMessage(dynamic_cast<TForm*>(Owner)->Handle, smObjectNameChanged, 0, (long)name_info);
   }

   if (d_co->ClassNameIs("TBusinessObject")) { // && rmInspector1->CurrentItem->ImageIndex > 1) {
      rmInspector1->CurrentItem->GetValueFromEditor(rmInspector1->Editor);
      TBusinessObjectField* bof = dynamic_cast<TBusinessObject*>(d_co)->getFields()->find(GetFieldNameFromPath(rmInspector1->CurrentItemPath));
      if (bof) {
         bof->setMapping(dynamic_cast<TrmStringInspectorItem*>(rmInspector1->CurrentItem)->Value);
         if (bof->getMapping().Length())
            rmInspector1->CurrentItem->ImageIndex = 0;
         else
            rmInspector1->CurrentItem->ImageIndex = 1;

         SendMessage(dynamic_cast<TForm*>(Owner)->Handle, smUpdateBOConnections, 0, 0);
/*
         TColor color = clLime;
         bool mappingfound = false;
         for (TBusinessObjectFieldCollection::CI bofi = dynamic_cast<TBusinessObject*>(d_co)->getFields()->begin();
                                                 bofi != dynamic_cast<TBusinessObject*>(d_co)->getFields()->end();
                                                 ++bofi) {
            if (!bofi->second->getMapping().Length())
               color = clRed;
            else
               mappingfound = true;
         }
         if (!mappingfound)
            color = clSilver;
         bool colorchanged = false;
         for (TConnectionCollection::CI cci = d_co->getConnections()->begin(); cci != d_co->getConnections()->end(); ++cci) {

            if (!cci->second->getOtherEnd()->ClassNameIs("TConnector"))
               continue;

            if (cci->second->getDirection() == cdOutbound)
               if (!mappingfound)
                  color = clRed;

            TConnection* conn1 = cci->second;
            TConnection* conn2 = cci->second->getOtherEnd()->getConnections()->findName(d_co->getName());
            if (conn2 && conn1->getGUIColor() != color) {
               conn1->setGUIColor(color);
               conn2->setGUIColor(color);
               colorchanged = true;
            }
         }

         if (colorchanged)
            SendMessage(dynamic_cast<TForm*>(Owner)->Handle, smRefreshConnections, 0, 0);
*/
         if (frmCommandScriptViewer)
            dynamic_cast<TfrmTransactionEditor*>(Owner)->GenerateCommandScript1Click(this);

      }

   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmObjectInspector::rmInspector1StartDrag(TObject *Sender,
      TDragObject *&DragObject)
{
   if (d_co->ClassNameIs("TBusinessObject")) {
      String PathString = rmInspector1->CurrentItemPath;
      String FieldName;
      TBusinessObject* bobject;
      TBusinessObjectFieldDragObject* bofdo = new TBusinessObjectFieldDragObject();
      bofdo->BusinessObject = dynamic_cast<TBusinessObject*>(d_co);
      bofdo->FieldName = GetFieldNameFromPath(PathString);
      DragObject = bofdo;
      return;
   }

   if (d_co->ClassNameIs("TConnector")) {
      String PathString = rmInspector1->CurrentItemPath;
      PathString.Delete(1, 9);
      String ObjectName;
      String FieldName;
      String MethodName;
      TConnectorObject* cobject;
      TConnectorObjectField* cfield = NULL;
      TServletMethod* cmethod = NULL;

      if (PathString.Pos(String(SEP_CHAR))) {
         ObjectName = PathString.SubString(1, PathString.Pos(String(SEP_CHAR))-1);
         if (rmInspector1->CurrentItem->ImageIndex == 2) {
            FieldName = GetFieldNameFromPath(PathString);
      //         FieldName = PathString.SubString(PathString.Pos(String(SEP_CHAR))+1, PathString.Length());
      //         FieldName.Delete(1, 7);
         }
         if (rmInspector1->CurrentItem->ImageIndex == 3) {
              MethodName = GetFieldNameFromPath(PathString);
      //         MethodName = PathString.SubString(PathString.Pos(String(SEP_CHAR))+1, PathString.Length());
      //         MethodName.Delete(1, 8);
         }
      }
      else
         ObjectName = PathString;

      TConnector* co = dynamic_cast<TConnector*>(d_co);
      cobject = co->getObjects()->find(ObjectName);
      if (cobject) {
         cfield =  cobject->getFields()->find(FieldName);
         cmethod = cobject->getMethods()->find(MethodName);
      }

      if (cobject && !cfield && !cmethod) {
         TConnectorObjectDragObject* codo = new TConnectorObjectDragObject();
         codo->ConnectorObject = cobject;
         codo->Connector = co;
         DragObject = codo;
         return;
      }

      if (cfield) {
         TConnectorObjectFieldDragObject* cofdo = new TConnectorObjectFieldDragObject();
         cofdo->ConnectorObjectField = cfield;
         cofdo->ConnectorObject = cobject;
         cofdo->Connector = co;
         DragObject = cofdo;
         return;
      }

      if (cmethod) {
         TServletMethodDragObject* comdo = new TServletMethodDragObject();
         comdo->ServletMethod = cmethod;
         comdo->ConnectorObject = cobject;
         comdo->Connector = co;
         DragObject = comdo;
         return;
      }
   }
   rmInspector1->EndDrag(false);
}
//---------------------------------------------------------------------------

void __fastcall TfrmObjectInspector::rmInspector1MouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   if (X < rmInspector1->SplitPos && (d_co->ClassNameIs("TConnector") || d_co->ClassNameIs("TBusinessObject")))
      rmInspector1->BeginDrag(false, 5);
}
//---------------------------------------------------------------------------

String __fastcall TfrmObjectInspector::GetFieldNameFromPath(const String& path)
{
   String tmp = path;
   tmp.Delete(1, tmp.Pos(String(SEP_CHAR)));
   tmp.Delete(1, tmp.Pos(String(SEP_CHAR)));
   return tmp;
}


void __fastcall TfrmObjectInspector::rmInspector1DragDrop(TObject *Sender,
      TObject *Source, int X, int Y)
{
   if (d_co->ClassNameIs("TBusinessObject")) {
      if (!Sender->ClassNameIs("TrmInspector")) {
         TPoint clientcoord = dynamic_cast<TControl*>(Sender)->ClientToScreen(TPoint(X, Y));
         TPoint parentcoord = rmInspector1->ScreenToClient(clientcoord);
         X = parentcoord.x;
         Y = parentcoord.y;
      }

      int Line = rmInspector1->TopIndex + (Y / rmInspector1->GridItemHeight);

      if (Source->ClassNameIs("TConnectorObjectFieldDragObject")) {
         rmInspector1->ItemIndex = Line;
         Application->ProcessMessages();
         TConnectorObjectFieldDragObject* cofdo = dynamic_cast<TConnectorObjectFieldDragObject*>(Source);
         String mapstring = cofdo->Connector->getName()+"."+cofdo->ConnectorObject->getName()+"."+cofdo->ConnectorObjectField->getName();
         rmInspector1->CurrentItem->AsString =  mapstring;
         TBusinessObjectField* bof = dynamic_cast<TBusinessObject*>(d_co)->getFields()->find(GetFieldNameFromPath(rmInspector1->CurrentItemPath));
         if (bof) {
            bof->setMapping(mapstring);
            if (bof->getMapping().Length())
               rmInspector1->CurrentItem->ImageIndex = 0;
            else
               rmInspector1->CurrentItem->ImageIndex = 1;
         }
         TColor color = clLime;
         bool mappingfound = false;
         for (TBusinessObjectFieldCollection::CI bofi = dynamic_cast<TBusinessObject*>(d_co)->getFields()->begin();
                                                 bofi != dynamic_cast<TBusinessObject*>(d_co)->getFields()->end();
                                                 ++bofi) {
            if (!bofi->second->getMapping().Length())
               color = clRed;
            else
               mappingfound = true;
         }

         TConnection* cnx = d_co->getConnections()->findName(cofdo->ConnectorObject->getParent()->getName());
         if (!cnx) {
            TConnection* connex = new TConnection();
            connex->setCode(connex->makeCode());
            connex->setName(d_co->getName());
            connex->setSource(cofdo->ConnectorObject->getParent());
            connex->setOtherEnd(d_co);
            connex->setDirection(cdOutbound);
            connex->setGUIColor(color);
            cofdo->ConnectorObject->getParent()->getConnections()->addItem(connex);

            TConnection* connex2 = new TConnection();
            connex2->setCode(connex2->makeCode());
            connex2->setName(cofdo->ConnectorObject->getParent()->getName());
            connex2->setSource(d_co);
            connex2->setOtherEnd(cofdo->ConnectorObject->getParent());
            connex2->setDirection(cdInbound);
            connex2->setGUIColor(color);
            d_co->getConnections()->addItem(connex2);

            SendMessage(dynamic_cast<TForm*>(Owner)->Handle, smRefreshConnections, 0, 0);
         }
         else  {
            for (TConnectionCollection::CI cnxi = d_co->getConnections()->begin(); cnxi != d_co->getConnections()->end(); ++cnxi) {
               TConnection* cnx = cnxi->second;
               
//               if (!cnx->ClassNameIs("TConnector"))
//                  continue;

               if (cnx->getDirection() == cdOutbound)
                  if (!mappingfound)
                     color = clRed;

               if (cnx->getGUIColor() != color) {
                  cnx->setGUIColor(color);
                  TConnection* conn2 = cofdo->ConnectorObject->getParent()->getConnections()->findName(d_co->getName());
                  if (conn2) {
                     conn2->setGUIColor(color);
                     SendMessage(dynamic_cast<TForm*>(Owner)->Handle, smRefreshConnections, 0, 0);
                  }
               }
            }
         }


      }
      /*
      if (Source->ClassNameIs("TServletMethodDragObject")) {
         rmInspector1->ItemIndex = Line;
         Application->ProcessMessages();
         TServletMethodDragObject* comdo = dynamic_cast<TServletMethodDragObject*>(Source);
         String mapstring = comdo->Connector->getName()+"."+comdo->ConnectorObject->getName()+"."+comdo->ServletMethod->getName();
         rmInspector1->CurrentItem->AsString = mapstring;
         TBusinessObjectField* bof = dynamic_cast<TBusinessObject*>(d_co)->getFields()->find(GetFieldNameFromPath(rmInspector1->CurrentItemPath));
         if (bof) {
            bof->setMapping(mapstring);
            if (bof->getMapping().Length())
               rmInspector1->CurrentItem->ImageIndex = 0;
            else
               rmInspector1->CurrentItem->ImageIndex = 1;
         }
         TColor color = clLime;
         for (TBusinessObjectFieldCollection::CI bofi = dynamic_cast<TBusinessObject*>(d_co)->getFields()->begin();
                                                 bofi != dynamic_cast<TBusinessObject*>(d_co)->getFields()->end();
                                                 ++bofi) {
            if (!bofi->second->getMapping().Length()) {
               color = clRed;
               break;
            }
         }
         TConnection* cnx = d_co->getConnections()->findName(comdo->ConnectorObject->getParent()->getName());
         if (!cnx) {
            TConnection* connex = new TConnection();
            connex->setCode(connex->makeCode());
            connex->setName(d_co->getName());
            connex->setSource(comdo->ConnectorObject->getParent());
            connex->setOtherEnd(d_co);
            connex->setDirection(cdOutbound);
            connex->setGUIColor(color);
            comdo->ConnectorObject->getParent()->getConnections()->addItem(connex);

            TConnection* connex2 = new TConnection();
            connex2->setCode(connex2->makeCode());
            connex2->setName(comdo->ConnectorObject->getParent()->getName());
            connex2->setSource(d_co);
            connex2->setOtherEnd(comdo->ConnectorObject->getParent());
            connex2->setDirection(cdInbound);
            connex2->setGUIColor(color);
            d_co->getConnections()->addItem(connex2);

            SendMessage(dynamic_cast<TForm*>(Owner)->Handle, smRefreshConnections, 0, 0);
         }
         else
            if (cnx->getGUIColor() != color) {
               cnx->setGUIColor(color);
               TConnection* conn2 = cnx->getOtherEnd()->getConnections()->findName(d_co->getName());
               if (conn2) {
                  conn2->setGUIColor(color);
                  SendMessage(dynamic_cast<TForm*>(Owner)->Handle, smRefreshConnections, 0, 0);
               }
            }
      }
      */
      if (frmCommandScriptViewer)
         dynamic_cast<TfrmTransactionEditor*>(Owner)->GenerateCommandScript1Click(this);
   }
}


//---------------------------------------------------------------------------
void __fastcall TfrmObjectInspector::rmInspector1DragOver(TObject *Sender,
      TObject *Source, int X, int Y, TDragState State, bool &Accept)
{
   if (d_ProcessingDrag) {
      Accept = false;
      return;
   }
   d_ProcessingDrag = true;
   Accept = false;
   if (!Sender->ClassNameIs("TrmInspector")) {
      if (rmInspector1->CurrentItemPath.Pos("Fields" + String(SEP_CHAR)))
         Accept = true;
      d_ProcessingDrag = false;
      return;
   }
   if (d_co->ClassNameIs("TBusinessObject") && (Source->ClassNameIs("TConnectorObjectFieldDragObject")/* || Source->ClassNameIs("TServletMethodDragObject")*/)) {
      int Line = rmInspector1->TopIndex + (Y / rmInspector1->GridItemHeight);
      if (Line < 0) {
         Accept = false;
         d_ProcessingDrag = false;
         return;
      }
      rmInspector1->ItemIndex = Line;
      Application->ProcessMessages();
      if (rmInspector1->CurrentItemPath.Pos("Fields")) {     // this'll expand the "Fields" parent; don't try to expand unexpandable children - strange things happen.
          if (!rmInspector1->CurrentItemPath.Pos("Fields" + String(SEP_CHAR))) {
             rmInspector1->CurrentNode->Expanded=true;
             Application->ProcessMessages();
          }
          else
             Accept = true;
      }
   }
   d_ProcessingDrag = false;
}

//---------------------------------------------------------------------------

void __fastcall TfrmObjectInspector::rmInspector1EditorExit(
      TObject *Sender)
{
   if (rmInspector1->CurrentItem->UserBool) {
      String oldvalue = rmInspector1->CurrentItem->AsString;
      rmInspector1->CurrentItem->GetValueFromEditor(rmInspector1->Editor);
      String newvalue = rmInspector1->CurrentItem->AsString;
      if (newvalue != oldvalue) {
         NameValueTyp r;
         r["sysClientId"] = "BusinessProcessModeller";
         r["sysRequest"] = "Set Param";
         r["sysConnectorName"] = d_co->getName().c_str();
         r["sysParamName"] = GetFieldNameFromPath(rmInspector1->CurrentItemPath).c_str();
         r["sysParamValue"] = newvalue.c_str();
         r["sysReply"] = "True";
         r["sysDebug"] = "True";

         SocketObj s(frmMain->ClientSocket1);
         try {
            s.Open();
            s << r;
            r.clear();
            s >> r;
            s.Close();
         }
         catch (...) {

         }
         if (r.find("sysOpResult") != r.end())
            StatusBar1->SimpleText = String(r["sysOpResult"].c_str()); // + " " + String(r["sysResultString"].c_str());
         if (String(r["sysRefresh"].c_str()) == "True") {
            r.clear();
            r["sysClientId"] = "BusinessProcessModeller";
            r["sysRequest"] = "Get Servlet Spec";
            r["sysConnectorName"] = d_co->getName().c_str();
            r["sysDebug"] = "True";
            r["sysReply"] = "True";
            try {
               s.Open();
               s << r;
               r.clear();
               s >> r;
               s.Close();
            }
            catch (...) { }

            dynamic_cast<TConnector*>(d_co)->getParams()->clearObjects();
            dynamic_cast<TConnector*>(d_co)->getObjects()->clearObjects();

            TStringList* fieldlist = new TStringList();
            TStringList* methodlist = new TStringList();
            std::map<String, String> paramlist;
            for (NameValueTyp::iterator cinfo = r.begin(); cinfo != r.end(); ++cinfo) {
               String temp = cinfo->first.c_str();
               if (String(cinfo->first.c_str()).Pos("Field"))
                  fieldlist->Add(cinfo->second.c_str());
               else
                  if (String(cinfo->first.c_str()).Pos("Method"))
                     methodlist->Add(cinfo->second.c_str());
                  else
                     if (String(cinfo->first.c_str()).Trim().Length() && String(cinfo->first.c_str()).Trim().Length() &&
                         String(cinfo->first.c_str()) != "sysClientId" &&
                         String(cinfo->first.c_str()) != "sysRequest" &&
                         String(cinfo->first.c_str()) != "sysConnectorName" &&
                         String(cinfo->first.c_str()) != "sysDebug" &&
                         String(cinfo->first.c_str()) != "sysReply" &&
                         String(cinfo->first.c_str()) != "sysResultStatus" &&
                         String(cinfo->first.c_str()) != "sysParamName")
                        paramlist[cinfo->first.c_str()] = cinfo->second.c_str();
            }
            TConnector* conn = dynamic_cast<TConnector*>(d_co);
            for (int i=0; i<=fieldlist->Count-1; i++) {
               String FieldItem = fieldlist->Strings[i];
               FieldItem.Delete(1, FieldItem.Pos("."));

               String ObjName = FieldItem.SubString(1, FieldItem.Pos(".")-1);
               TConnectorObject* connobj = conn->getObjects()->find(ObjName);
               if (!connobj) {
                  connobj = new TConnectorObject(conn, ObjName, ObjName);
                  conn->getObjects()->addItem(connobj);
               }

               String fieldname = FieldItem.SubString(FieldItem.Pos(".")+1, FieldItem.Length());
               connobj->getFields()->addItem(new TConnectorObjectField(fieldname, fieldname));
               Application->ProcessMessages();
            }

            for (int i=0; i<=methodlist->Count-1; i++) {
               String MethodItem = methodlist->Strings[i];
               MethodItem.Delete(1, MethodItem.Pos("."));

               String ObjName = MethodItem.SubString(1, MethodItem.Pos(".")-1);
               TConnectorObject* connobj = conn->getObjects()->find(ObjName);
               if (!connobj) {
                  connobj = new TConnectorObject(conn, ObjName, ObjName);
                  conn->getObjects()->addItem(connobj);
               }

               String methodname = MethodItem.SubString(MethodItem.Pos(".")+1, MethodItem.Length());
               connobj->getMethods()->addItem(new TServletMethod(methodname, methodname));
               Application->ProcessMessages();
            }

            for (std::map<String, String>::iterator pi = paramlist.begin(); pi != paramlist.end(); ++ pi) {
               TServletParam* cp = new TServletParam(pi->first, pi->second);
               conn->getParams()->addItem(cp);
            }

            delete fieldlist;
            delete methodlist;
            s.Close();
            rebuild(d_co);
            SetFocus();
         }
      }
   }
}
//---------------------------------------------------------------------------



void __fastcall TfrmObjectInspector::SMUpdateContentsHandler(TMessage& msg)
{
   if (d_co->ClassNameIs("TBusinessObject")) { // && rmInspector1->CurrentItem->ImageIndex > 1) {
      TBusinessObject* bo = dynamic_cast<TBusinessObject*>(d_co);
      for (TBusinessObjectFieldCollection::CI bofi = bo->getFields()->begin(); bofi != bo->getFields()->end(); ++bofi) {
         TrmStringInspectorItem* ii = dynamic_cast<TrmStringInspectorItem*>(rmInspector1->FindInspectorItem(String(SEP_CHAR)+"Fields"+String(SEP_CHAR)+bofi->second->getName().Trim()));
         if (ii) {
            ii->Value = bofi->second->getMapping();
            if (bofi->second->getMapping().Length())
               ii->ImageIndex = 0;
            else
               ii->ImageIndex = 1;
         }
      }
      SendMessage(dynamic_cast<TForm*>(Owner)->Handle, smUpdateBOConnections, 0, 0);
   }
   if (d_co->ClassNameIs("TTransaction")) {
      TTransaction* to = dynamic_cast<TTransaction*>(d_co);
      TrmStringInspectorItem* ii = dynamic_cast<TrmStringInspectorItem*>(rmInspector1->FindInspectorItem(String(SEP_CHAR)+"Name"));
      if (ii)
         ii->Value = to->getName();
      Caption = "Inspecting " + to->getName();
   }
}
void __fastcall TfrmObjectInspector::rmInspector1ItemIndexChanging(
      TrmTreeNonViewNode *CurrentNode, TrmTreeNonViewNode *NewNode,
      bool &AllowChange)
{
   if (rmInspector1->Editor) {
      rmInspector1EditorExit(rmInspector1->Editor);
   }
   AllowChange = true;}
//---------------------------------------------------------------------------

