//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "OBJBusinessObject.h"
#include "OBJBusinessObjectField.h"
#include "OBJConnector.h"
#include "OBJConnectorObject.h"
#include "OBJConnectorObjectField.h"
#include "OBJServletMethod.h"
#include "OBJConnectableObject.h"
#include "OBJServletParam.h"
#include "OBJSocket.h"

#include "UIAnimateLoad.h"
#include "UIMain.h"
#include "UIObjectInspector2.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "dbinsp"
#pragma link "kbmMemTable"
#pragma link "oinspect"
#pragma link "Placemnt"
#pragma resource "*.dfm"
TfrmObjectInspector2 *frmObjectInspector2;

enum TInsepectorItemType {iitBusinessObjectField=1, iitServletMethod, iitServletParam, iitConnectorObjectField};

//---------------------------------------------------------------------------
__fastcall TfrmObjectInspector2::TfrmObjectInspector2(TComponent* Owner, TConnectableObject* co)
        : TForm(Owner)
{
   d_co = co;
   Caption = "Inspecting " + d_co->getName();
   rebuild(co);
}
//---------------------------------------------------------------------------


void __fastcall TfrmObjectInspector2::rebuild(TConnectableObject* co)
{
   DataSetInspector1->BeginUpdate();
   DataSetInspector1->Colors->BackColor = co->getGUIColor();
   DataSetInspector1->Colors->HighlightBkgnd = co->getGUIColor();
   if (dsMemory->Active)
      dsMemory->EmptyTable();
   dsMemory->Active = false;
   DataSetInspector1->Items->Clear();
   DataSetInspector1->Aliases->Clear();

   TStringField* field = new TStringField(NULL);

   field->FieldName =  "Connections";
   dsMemory->CreateFieldAs(field);
   TInspectorItem* dii = dynamic_cast<TInspectorItem*>(DataSetInspector1->Items->Add());
   dii->FieldName = "Connections";
   dii->Title = "Connections";
   dii->ReadOnly = true;
   dii->ButtonStyle = ibsEllipsis;
   dii->Tag = 0;

   field->FieldName = "ObjectName";
   dsMemory->CreateFieldAs(field);
   dii = dynamic_cast<TInspectorItem*>(DataSetInspector1->Items->Add());
   dii->FieldName = "ObjectName";
   dii->Title = "Object Name";
   dii->ReadOnly = false;
   dii->ButtonStyle = ibsNone;
   dii->Tag = 0;

   if (co->ClassNameIs("TBusinessObject")) {
      TBusinessObject* bo = dynamic_cast<TBusinessObject*>(co);

      DataSetInspector1->Aliases->Add("[Fields]");

      for (TBusinessObjectFieldCollection::CI boi = bo->getFields()->begin(); boi != bo->getFields()->end(); ++boi) {
         DataSetInspector1->Aliases->Add(boi->first);
         field->FieldName = boi->first;
         dsMemory->CreateFieldAs(field);
         TInspectorItem* dii = dynamic_cast<TInspectorItem*>(DataSetInspector1->Items->Add());
         dii->FieldName = boi->first;
         dii->Title = boi->first;
         dii->ReadOnly = false;
         dii->ButtonStyle = ibsNone;
         dii->Tag = iitBusinessObjectField;

      }
      DataSetInspector1->Aliases->Add("");
   }

   if (co->ClassNameIs("TConnector")) {
      TConnector* conn = dynamic_cast<TConnector*>(co);

      for (TConnectorObjectCollection::CI coi = conn->getObjects()->begin(); coi != conn->getObjects()->end(); ++coi) {
         DataSetInspector1->Aliases->Add("["+coi->second->getName()+"]");

         for (TConnectorObjectFieldCollection::CI cofi = coi->second->getFields()->begin(); cofi != coi->second->getFields()->end(); ++cofi) {
            String str = cofi->second->getName();
            while (str.Pos("."))
               str.Delete(str.Pos("."), 1);
            str = coi->second->getName()+str;

            if (!dsMemory->Fields->FindField(str)) {
               DataSetInspector1->Aliases->Add(coi->second->getName()+"."+cofi->second->getName());
               field->FieldName = str;
               dsMemory->CreateFieldAs(field);
               TInspectorItem* dii = dynamic_cast<TInspectorItem*>(DataSetInspector1->Items->Add());
               dii->FieldName = str;
               dii->Title = coi->second->getName()+"."+cofi->second->getName();
               dii->ReadOnly = true;
               dii->ButtonStyle = ibsNone;
               dii->Tag = iitConnectorObjectField;
            }
         }

         for (TServletMethodCollection::CI comi = coi->second->getMethods()->begin(); comi != coi->second->getMethods()->end(); ++comi) {
            String str = comi->second->getName();
            while (str.Pos("."))
               str.Delete(str.Pos("."), 1);
            str = coi->second->getName()+str;
            if (!dsMemory->Fields->FindField(str)) {
               DataSetInspector1->Aliases->Add(coi->second->getName()+"."+comi->second->getName());
               field->FieldName = str;
               dsMemory->CreateFieldAs(field);
               TInspectorItem* dii = dynamic_cast<TInspectorItem*>(DataSetInspector1->Items->Add());
               dii->FieldName = str;
               dii->Title = coi->second->getName()+"."+comi->second->getName();
               dii->ReadOnly = true;
               dii->ButtonStyle = ibsNone;
               dii->Tag = iitServletMethod;
            }
         }

         DataSetInspector1->Aliases->Add("");

      }

      DataSetInspector1->Aliases->Add("[Parameters]");

      for (TServletParamCollection::CI spi = conn->getParams()->begin(); spi != conn->getParams()->end(); ++spi) {
         if (spi->second->getName().Trim().Length()) {
            if (spi->first.Pos("sysConfigParamName")) {
               String paramNbr = spi->first.SubString(19, 4);
               String ParamAllow = "sysParamAllow" + paramNbr;
               String ParamDisplay = "sysParamDisplay" + paramNbr;
               String ParamSize = "sysParamSize" + paramNbr;

               DataSetInspector1->Aliases->Add(spi->second->getName());
               field->FieldName = spi->second->getName();
               TServletParam* pSize = conn->getParams()->find(ParamSize);
               if (pSize)
                  field->Size = min(abs(StrToIntDef(pSize->getName(), 255)), 255);

               dsMemory->CreateFieldAs(field);
               field->Size = 255;
               TInspectorItem* dii = dynamic_cast<TInspectorItem*>(DataSetInspector1->Items->Add());
               dii->FieldName = spi->second->getName();
               dii->Title = spi->second->getName();
               dii->ReadOnly = false;
               dii->ButtonStyle = ibsNone;
               dii->Tag = iitServletParam;
            }
         }
      }
      DataSetInspector1->Aliases->Add("");
   }
   dsMemory->Active = true;
   dsMemory->Insert();
   dsMemory->FieldByName("ObjectName")->AsString = co->getName();

   if (co->ClassNameIs("TConnector")) {
      TConnector* conn = dynamic_cast<TConnector*>(co);

      std::auto_ptr<TfrmAnimateLoad> al(new TfrmAnimateLoad(this));
      al->Show();

      NameValueTyp r;
      r["sysClientId"] = "BusinessProcessModeller";
      r["sysRequest"] = "Get Config Values";
      r["sysConnectorName"] = co->getName().c_str();
      r["sysReply"] = "True";
      r["sysDebug"] = "True";

      SocketObj s(frmMain->ClientSocket1);
      try {
         s.Open();
         s << r;
         r.clear();
         s >> r;
         s.Close();
      }
      catch (...) {

      }

      for (TServletParamCollection::CI spi = conn->getParams()->begin(); spi != conn->getParams()->end(); ++spi)
         if (spi->second->getName().Trim().Length())
            if (spi->first.Pos("sysConfigParamName")) {
               String paramNbr = spi->first.SubString(19, 4);
               NameValueTyp::iterator val = r.find(String("sysConfigParamValue"+paramNbr).c_str());
               String valuestr = "";
               if (val != r.end())
                  valuestr = String(val->second.c_str());

               dsMemory->FieldByName(spi->second->getName())->AsString = valuestr;
            }
   }
   dsMemory->Post();
   DataSetInspector1->Options = DataSetInspector1->Options << oiUseAliases;
   DataSetInspector1->Options = DataSetInspector1->Options << oiUseGroups;
   DataSetInspector1->EndUpdate();
   
   if (co->ClassNameIs("TConnector")) {
      TConnector* conn = dynamic_cast<TConnector*>(co);
      for (TConnectorObjectCollection::CI coi = conn->getObjects()->begin(); coi != conn->getObjects()->end(); ++coi)
         DataSetInspector1->CollapseGroupByName(coi->second->getName());
   }
}

TInspectorItem* __fastcall TfrmObjectInspector2::findInspectorItem(const String& Title)
{
   for (int i = 0; i <= DataSetInspector1->Items->Count-1; i++)
      if (DataSetInspector1->Items->Items[i]->Title == Title)
         return DataSetInspector1->Items->Items[i];

   return NULL;
}



void __fastcall TfrmObjectInspector2::DataSetInspector1DrawCell(
      TObject *Sender, TPropertyEditor *PropEdit, int Column, TRect &ARect,
      bool &Processed)
{
   Processed = false;
   String propName = PropEdit->GetName();
   String propValue = PropEdit->GetValue();
   if (Column == 0) {
      if (d_co->ClassNameIs("TBusinessObject")) {

         TInspectorItem* ii = findInspectorItem(propName);
         if (ii && ii->Tag == iitBusinessObjectField) {
            DataSetInspector1->Canvas->FillRect(ARect);
            std::auto_ptr<Graphics::TBitmap> bmp(new Graphics::TBitmap());
            if (!propValue.Length())
               bmp->Assign(redimg->Picture->Bitmap);
            else
               bmp->Assign(greenimg->Picture->Bitmap);
            DataSetInspector1->Canvas->BrushCopy(Bounds(ARect.Left+2, ARect.Top, bmp->Width, bmp->Height), bmp.get(), Bounds(0, 0, bmp->Width, bmp->Height), clFuchsia);
            DataSetInspector1->Canvas->TextOut(ARect.Left+bmp->Width+6, ARect.Top+2, propName);
            Processed = true;
         }
      }

      if (d_co->ClassNameIs("TConnector")) {
         TInspectorItem* ii = findInspectorItem(propName);
         if (ii) {
            std::auto_ptr<Graphics::TBitmap> bmp(new Graphics::TBitmap());
            switch (ii->Tag) {
               case iitConnectorObjectField:
                  DataSetInspector1->Canvas->FillRect(ARect);
                  bmp->Assign(fieldimg->Picture->Bitmap);
                  DataSetInspector1->Canvas->BrushCopy(Bounds(ARect.Left+2, ARect.Top, bmp->Width, bmp->Height), bmp.get(), Bounds(0, 0, bmp->Width, bmp->Height), clGray);
                  DataSetInspector1->Canvas->TextOut(ARect.Left+bmp->Width+6, ARect.Top+2, propName);
                  Processed = true;
                  break;
               case iitServletMethod:
                  DataSetInspector1->Canvas->FillRect(ARect);
                  bmp->Assign(methodimg->Picture->Bitmap);
                  DataSetInspector1->Canvas->BrushCopy(Bounds(ARect.Left+2, ARect.Top, bmp->Width, bmp->Height), bmp.get(), Bounds(0, 0, bmp->Width, bmp->Height), clSilver);
                  DataSetInspector1->Canvas->TextOut(ARect.Left+bmp->Width+6, ARect.Top+2, propName);
                  Processed = true;
                  break;
            }
         }
      }
   }
}
//---------------------------------------------------------------------------



